-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

libOpen ("xbee.so.1.0","/dev/ttyS0")
thread("utils/carloGavazzi")
thread("apps/mywatts/mywattsmqtt")
thread("apps/mywatts/mywattsdigi")
local pirTime = 5000;

-- Define addresses of radios we will use

local radioBase        = "0013A20040B0994B"
local radioRemoteRelay = "0013A20040B097C0"
local radioLocalRelay  = "0013A20040A1449F"
local radioNode1       = "0013A20040B09786" -- Normal
local radioNode4       = "0013A20040A14D0B" -- confirmed GOOD
local radioNode2       = "0013A20040A14CFA"
local radioNode3       = "0013A2004079E633" -- confirmed GOOD


local Mode_Normal = 1
local Mode_Night = 2
local Mode_Holiday = 3
local Mode_MaxDemand = 4
local relayAllOff = "n"

output = {}
output["xbee.node1.pir.timer"] = 1
output["xbee.node2.pir.timer"] = 2
output["xbee.node3.pir.timer"] = 3
output["xbee.node4.pir.timer"] = 4
output["xbee.node1.pir"] = 1
output["xbee.node2.pir"] = 2
output["xbee.node3.pir"] = 3
output["xbee.node4.pir"] = 4

output["alarm"]           = 1
output["mains.Lab_light"] = 2
output["mains.Lab_fan"]   = 3
output["mains.device3"]   = 4

function updateMainsDevice (tag)
    local event = string.gsub(tag,".event","");
    print ("Update mains device " .. event)
    local xbee = require "utils/xbee"    


    dev = "mains." .. event
    out = output[dev]
    val = tagGetNum (event)

    print ("setting " .. out .. " from " .. dev .. " to " .. val)

    xbee.setRelay (radioLocalRelay, out, val)
end

function pirOffTimer (event)
    print ("PIR: off timer for " .. event)
    local xbee = require "utils/xbee"    

    -- pir = strsub(event, 1, 14)
    pir = event
    print ("PIR: get output for " .. pir)
    out = output[pir]

    print ("PIR: setting " .. out .. " from " .. pir .. " to off")

-- for testing, turn all off
-- xbeeSendData (radioRemoteRelay, relayAllOff)

    xbee.setRelay (radioRemoteRelay, out, 0)
end

function pirChange (tag)
    local name = string.gsub(tag,".event","");
    local pir = tagGetNum (name)
    local xbee = require "utils/xbee"    
    print ("tag " .. name .. " updated to " .. pir)

    -- Get the mode.  1 is normal, 2 is night, 3 is holiday
    local mode = tagGetNum ("House_mode")

    if (pir > 0) then
        if (mode == Mode_Holiday) then
            -- If motion detected and we are in holiday mode then
            -- activate the alarm
            alarm = output["alarm"]
            xbee.setRelay (radioLocalRelay, alarm, 1)
        else
            -- Otherwise, turn on the light and stop the off timer to turn
            -- it off again.
            print ("PIR: get output for " .. pir)
            out = output[name]
            timerStop (name .. "timer")
            xbee.setRelay (radioRemoteRelay, out, 1)
            print ("PIR: set " .. out .. " from " .. name .. " to " .. pir)
        end
    else
        if (mode == Mode_Holiday) then
            print ("PIR: off in holiday mode, ignore")
        else
            -- start the off timer to turn
            -- it off again.
            print ("PIR: start off timer")
            timerStart (name .. ".timer", pirTime)
        end
    end
end


function tempChange (tag)

    local name = string.gsub(tag,".event","");
    print ("tag " .. name .. " is " .. tagGetNum(name))
    local xbee = require "utils/xbee"    

    temp = xbee.temp(name)

    if (name == "xbee.node1.temp") then
        print ("temp 1 " .. temp)
        tagSetNum ("Bedroom", temp)
    elseif (name == "xbee.node2.temp") then
        print ("temp 2 " .. temp)
        tagSetNum ("Boxroom", temp)
    elseif (name == "xbee.node3.temp") then
        print ("temp 3 " .. temp)
        tagSetNum ("Bathroom", temp)
    elseif (name == "xbee.node4.temp") then
        print ("temp 4 " .. temp)
        tagSetNum ("Lounge", temp)
   end
end

function modeChange (tag)
    local name = string.gsub(tag,".event","");
    print ("************************\ntag " .. name .. " is " .. tagGetNum(name))
    local xbee = require "utils/xbee"    
    val = tagGetNum (name)

    if (val == Mode_Normal) then
        -- In normal mode, turn off the alarm and set all appliance
        -- settings to their tag values
        out = output["alarm"]
        xbee.setRelay (radioLocalRelay, out, 0)
        tagSetString ("Lab_light", "1")
        tagSetString ("Lab_fan", "1")

--        updateMainsDevice ("Lab_light")
--        updateMainsDevice ("Lab_fan")
--        updateMainsDevice ("device3")
--        tagSetNum ("Lab_light", 1)
--        tagSetNum ("Lab_fan", 1)
    elseif (val == Mode_Holiday) then
        -- In holiday mode turn everything off



        xbeeSendData (radioLocalRelay, relayAllOff)
        xbeeSendData (radioRemoteRelay, relayAllOff)
        tagSetString ("Lab_light", "0")
        tagSetString ("Lab_fan", "0")
    end
end

function init ()
    -- start the step timer
    timerStart ("xbee.timer", 1000);

    -- turn off all relays
    xbeeSendData (radioLocalRelay, relayAllOff)
    xbeeSendData (radioRemoteRelay, relayAllOff)
   -- set normal mode
    tagSetNum ("House_mode", Mode_Normal)
end


-- define a new xbee device by 64-bit address, with tag xbee.remote1
xbeeDefineDataDevice (radioLocalRelay, "xbee.local-relay");
xbeeDefineDataDevice (radioRemoteRelay, "xbee.remote-relay");
xbeeDefineIoDevice (radioNode1, "xbee.node1");
xbeeDefineIoDevice (radioNode2, "xbee.node2");
xbeeDefineIoDevice (radioNode3, "xbee.node3");
xbeeDefineIoDevice (radioNode4, "xbee.node4");

-- define digital pin 3 on the remote nodes as digital pir inputs
xbeeDefineDigitalInput (radioNode1, 3, 0, "xbee.node1.pir");
xbeeDefineDigitalInput (radioNode2, 3, 0, "xbee.node2.pir");
xbeeDefineDigitalInput (radioNode3, 3, 0, "xbee.node3.pir");
xbeeDefineDigitalInput (radioNode4, 3, 0, "xbee.node4.pir");

-- define digital pin 2 on the remote nodes as analog temperature inputs
xbeeDefineAnalogInput (radioNode1, 2, 0, "xbee.node1.temp");
xbeeDefineAnalogInput (radioNode2, 2, 0, "xbee.node2.temp");
xbeeDefineAnalogInput (radioNode3, 2, 0, "xbee.node3.temp");
xbeeDefineAnalogInput (radioNode4, 2, 0, "xbee.node4.temp");

-- Set up traps for any change of any pir sensor
tagSetEvent ("xbee.node1.pir", "xbee.node1.pir.event");
eventHandler ("xbee.node1.pir.event", pirChange);
tagSetEvent ("xbee.node2.pir", "xbee.node2.pir.event");
eventHandler ("xbee.node2.pir.event", pirChange);
tagSetEvent ("xbee.node3.pir", "xbee.node3.pir.event");
eventHandler ("xbee.node3.pir.event", pirChange);
tagSetEvent ("xbee.node4.pir", "xbee.node4.pir.event");
eventHandler ("xbee.node4.pir.event", pirChange);

-- Set up traps for any change of any temp sensor
tagSetEvent ("xbee.node1.temp", "xbee.node1.temp.event");
eventHandler ("xbee.node1.temp.event", tempChange);
tagSetEvent ("xbee.node2.temp", "xbee.node2.temp.event");
eventHandler ("xbee.node2.temp.event", tempChange);
tagSetEvent ("xbee.node3.temp", "xbee.node3.temp.event");
eventHandler ("xbee.node3.temp.event", tempChange);
tagSetEvent ("xbee.node4.temp", "xbee.node4.temp.event");
eventHandler ("xbee.node4.temp.event", tempChange);

-- timers for PIR off to debounce the PIR inputs
eventHandler ("xbee.node1.pir.timer", pirOffTimer);
eventHandler ("xbee.node2.pir.timer", pirOffTimer);
eventHandler ("xbee.node3.pir.timer", pirOffTimer);
eventHandler ("xbee.node4.pir.timer", pirOffTimer);

-- Trap any updates to status of mains powered devices
tagSetEvent ("Lab_light", "Lab_light.event")
eventHandler ("Lab_light.event", updateMainsDevice)

tagSetEvent ("Lab_fan", "Lab_fan.event")
eventHandler ("Lab_fan.event", updateMainsDevice)

tagSetEvent ("device3", "device3.event")
eventHandler ("device3.event", updateMainsDevice)
tagSetEvent ("House_mode", "House_mode.event")
eventHandler ("House_mode.event", modeChange)

-- assign a timer handler
eventHandler ("xbee.timer", step);

-- start the init timer.  Give it 1 second to settle down before
-- we start setting up initial values
eventHandler ("home.init.timer", init);
timerStart ("home.init.timer", 1000);



while 1 do
    local event = eventWait ();
    print ("got event " .. event);
    eventDispatch (event);
end
