-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local params = {...}
local mqttClient = params[1]
local vendorID = params[2]
local deviceID = params[3]

local config = require "utils/config"

local vendingSensor = {
    --Ssp sensors
    cashAcepted       = "cashAcepted",
    cashAceptorStatus = "cashAceptorStatus",
    cashDispensed     = "cashDispensed",
    itemDispensed     = "itemDispensed",
    motorRunning      = "motorRunning",
    --Local input sensors, reports rules calculated data
    accumulatedCash   = "accumulatedCash",
    change            = "change",
    --Other input sensors
    pressedKey        = "pressedKey",
    --Local output sensors, reports data from this thread
    state             = "vendingState",
    selectedItem      = "selectedItem",
    selectedItemPrice = "selectedItemPrice",
    vendingLog        = "log"
}

local vendingActuator = {
    --Ssp actuators
    cashAcceptance = "cashAcceptance",
    cashDispenser  = "cashDispenser",
    itemDispenser  = "itemDispenser",
    display        = "display",
    --Local input actuator, allows sending actions to this thread
    restart        = "vendingRestart"
}

local vendingState = {
    start = 0,
    waiting = 1,
    aceptingCash = 2,
    correctAmount = 3,
    dispensingChange = 4,
    dispensingItem = 5,
    finish = 6
}

local itemPrice = 0
local accumulatedCash = 0
local selectedItem = 0
local transaction = 0
local nextTransaction = 1
local cashAceptorStatus = 0

local vendingCurrState = vendingState.start

local function printCurrentVariables()
    NOTIFY ('accumulatedCash   = ' .. accumulatedCash)
    NOTIFY ('cashAceptorStatus = ' .. cashAceptorStatus)
    NOTIFY ('selectedItem      = ' .. selectedItem)
    NOTIFY ('itemPrice         = ' .. itemPrice)
    NOTIFY ('transaction       = ' .. transaction)
    NOTIFY ('nextTransaction   = ' .. nextTransaction)
end

local function updateDisplay()
    local cmd = string.char(0xFE,0x58)

    if vendingCurrState == vendingState.start then
        cmd = cmd .. "  Starting...   "
    elseif vendingCurrState == vendingState.waiting then
        cmd = cmd .. " Select an item "
        cmd = cmd .. "    please...   "
    elseif vendingCurrState == vendingState.aceptingCash then
        cmd = cmd .. "  Insert coins  "

        local letter = 65 + math.floor(selectedItem/10)
        if letter > 72 then
            letter = letter + 1
        end
        number = math.fmod(selectedItem,10)

        cmd = cmd .. string.format("%s%02d    %5.2f EUR", string.char(letter), number, itemPrice - accumulatedCash)

    elseif vendingCurrState == vendingState.correctAmount then
        cmd = cmd .. "  Dispensing    "
        cmd = cmd .. "   change...    "
    elseif vendingCurrState == vendingState.dispensingChange then
        cmd = cmd .. "  Dispensing    "
        cmd = cmd .. "   change...    "
    elseif vendingCurrState == vendingState.dispensingItem then
        cmd = cmd .. "  Dispensing    "
        cmd = cmd .. "    item...     "
    elseif vendingCurrState == vendingState.finish then
        cmd = cmd .. "  Finishing...  "
    end

    tagSetString (vendingActuator.display, cmd)
end

local function setVendingState(state)
    if state == vendingState.start then
        NOTIFY ('State = Start')
    elseif state == vendingState.waiting then
        NOTIFY ('State = Waiting')
    elseif state == vendingState.aceptingCash then
        NOTIFY ('State = Acepting cash')
    elseif state == vendingState.correctAmount then
        NOTIFY ('State = Correct amount')
    elseif state == vendingState.dispensingChange then
        NOTIFY ('State = Dispensing change')
    elseif state == vendingState.dispensingItem then
        NOTIFY ('State = Dispensing item')
    elseif state == vendingState.finish then
        NOTIFY ('State = Finish')
    end

    vendingCurrState = state

    updateDisplay()

    tagSetNum (vendingSensor.state, state)
end

local function start(tag ,value)
    if cashAceptorStatus ~= 1 then
        tagSetNum (vendingActuator.cashAcceptance, -1)
    end

    itemPrice = 0
    selectedItem = 0
    transaction = 0
    nextTransaction = 1
    cashAceptorStatus = 0

    setVendingState(vendingState.waiting)
end

local function waiting(tag ,value)
    if tag == vendingSensor.selectedItem then
        selectedItem = tonumber(value)
        local price = config.getValue(vendorID, deviceID, string.format("price_%d", selectedItem), "0")
        itemPrice = tonumber(price)
        tagSetString (vendingSensor.selectedItemPrice, price)
    end

    if cashAceptorStatus > 0 and selectedItem ~= 0 and itemPrice > 0 then
        tagSetNum (vendingActuator.cashAcceptance, 1)
        setVendingState(vendingState.aceptingCash)
    end
end

local function aceptingCash(tag ,value)
    if tag == vendingSensor.selectedItem then
        local price = config.getValue(vendorID, deviceID, string.format("price_%d", tonumber(value)), "0")
        if tonumber(price) > 0 then
            selectedItem = tonumber(value)
            itemPrice = tonumber(price)
            tagSetString (vendingSensor.selectedItemPrice, price)
        end
    end

    if accumulatedCash >= itemPrice then
        setVendingState(vendingState.correctAmount)
    end
end

local function correctAmount(tag ,value)
    local change = accumulatedCash - itemPrice
    if change > 0 then
        tagSetNum (vendingActuator.cashDispenser, change)
    else
        tagSetNum (vendingActuator.cashAcceptance, -1)
    end
    setVendingState(vendingState.dispensingChange)
end

local function dispensingChange(tag ,value)
    if cashAceptorStatus > 0 then
        tagSetNum (vendingActuator.itemDispenser, selectedItem)
        setVendingState(vendingState.dispensingItem)
    end
end

local function dispensingItem(tag ,value)
    if transaction >= nextTransaction then
        nextTransaction = transaction + 1
        setVendingState(vendingState.finish)
    end
end

local function setVendingLog()
    local sku = config.getValue(vendorID, deviceID, string.format("sku_%d", selectedItem), "")
    local vendingLog = string.format("{\"slot\":%d,\"price\":%f,\"sku\":\"%s\"}", selectedItem, itemPrice, sku)
    tagSetString (vendingSensor.vendingLog, vendingLog)
end

local function finish(tag ,value)
    if cashAceptorStatus == 1 then
        setVendingLog()
        accumulatedCash = 0
        setVendingState(vendingState.start)
    end
end

local function vendingHandler (tag ,value)

    local nValue = tonumber(value)

    if tag == vendingSensor.cashAceptorStatus then
        cashAceptorStatus = nValue
        updateDisplay()
    elseif (tag == vendingSensor.itemDispensed) and (nValue ~= 0) then
        if vendingCurrState ~= vendingState.dispensingItem then
            NOTIFY ('Restarting...')
            setVendingLog()
            setVendingState(vendingState.start)
        else
            transaction = transaction + 1
        end
        printCurrentVariables()
    elseif (tag == vendingSensor.cashDispensed) and (nValue ~= 0) and
           (vendingCurrState < vendingState.dispensingChange) then
        NOTIFY ('Restarting...')
        setVendingState(vendingState.start)
        printCurrentVariables()
    elseif tag == vendingSensor.accumulatedCash then
        accumulatedCash = nValue
        printCurrentVariables()
    elseif (tag == vendingActuator.restart) and (nValue ~= 0) then
        NOTIFY ('Restarting...')
        setVendingState(vendingState.start)
    end

    if vendingCurrState == vendingState.start then
        start(tag ,nValue)
    elseif vendingCurrState == vendingState.waiting then
        waiting(tag ,nValue)
    elseif vendingCurrState == vendingState.aceptingCash then
        aceptingCash(tag ,nValue)
    elseif vendingCurrState == vendingState.correctAmount then
        correctAmount(tag ,nValue)
    elseif vendingCurrState == vendingState.dispensingChange then
        dispensingChange(tag ,nValue)
    elseif vendingCurrState == vendingState.dispensingItem then
        dispensingItem(tag ,nValue)
    elseif vendingCurrState == vendingState.finish then
        finish(tag ,nValue)
    end

    return true
end

local key = {}
key[0]  = 15    --A
key[1]  = 29    --B
key[2]  = 43    --C
key[3]  = 52    --D
key[4]  = 62    --E
key[5]  = 70    --F
key[6]  = 78    --G
key[7]  = 86    --H
key[8]  = 93    --J
key[9]  = 98    --K
key[10] = 104   --1
key[11] = 109   --2
key[12] = 114   --3
key[13] = 120   --4
key[14] = 125   --5
key[15] = 127   --6
--key[16] = 131   --7
--key[17] = 135   --8
--key[18] = 138   --9
--key[19] = 140   --10

local keyValues = {}
keyValues[0]  = 0   --A
keyValues[1]  = 10  --B
keyValues[2]  = 20  --C
keyValues[3]  = 30  --D
keyValues[4]  = 40  --E
keyValues[5]  = 50  --F
keyValues[6]  = 60  --G
keyValues[7]  = 70  --H
keyValues[8]  = 80  --J
keyValues[9]  = 90  --K
keyValues[10] = 1
keyValues[11] = 2
keyValues[12] = 3
keyValues[13] = 4
keyValues[14] = 5
keyValues[15] = 6
keyValues[16] = 7
keyValues[17] = 8
keyValues[18] = 9
keyValues[19] = 10

local letterKey = -1
local numberKey = -1
local oldValue = 255

local oldI = 100

local stabilityCnt = 0

local function keyboardHandler (tag ,value)
    oldValue = tonumber(value)

    for i = 0, 15, 1
    do
        if oldValue <= key[i] then
            if i == oldI then
                stabilityCnt = stabilityCnt + 1
            else
                stabilityCnt = 0
            end
            oldI = i

            if stabilityCnt > 1 then
                NOTIFYF ('Pressed key %d (%d)', i, oldValue)
                if i < 10 then
                    letterKey = keyValues[i]
                    numberKey = -1
                else
                    numberKey = keyValues[i]
                end
            end
            break
        end
    end

    if (letterKey >= 0) and (numberKey >= 0) then
        NOTIFYF ('Selected item %d', letterKey + numberKey)
        tagSetNum (vendingSensor.selectedItem, letterKey + numberKey)
        letterKey = -1
        numberKey = -1
    end

    return true
end

-- MAIN
eventHandler (vendingSensor.accumulatedCash,   vendingHandler)
eventHandler (vendingSensor.selectedItem,      vendingHandler)
eventHandler (vendingSensor.itemDispensed,     vendingHandler)
eventHandler (vendingSensor.cashDispensed,     vendingHandler)
eventHandler (vendingSensor.cashAceptorStatus, vendingHandler)
eventHandler (vendingSensor.pressedKey,        keyboardHandler)
eventHandler (vendingActuator.restart,         vendingHandler)

NOTIFY ('vending thread started')

while true do
    local event, param = eventWait ()
    eventDispatch (event, param)
end
