-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--! \file
--! \brief This is the weather client module

-- This module reads values from a locally attached sensor
-- and publishes the data using MQTT

--[[
speed_values = {}
direction_values = {}

count = 0;
max = 0;
]]

local hwaddr="11:22:33:44:55:66"
local device_topic="device_" .. hwaddr
local server_topic = "server_" .. hwaddr
local sensor_is_active = 0
local location = "latitude=52.555555,longitude=-8.888888"
local topic_prefix = "com.emutexlabs.weather."


function data_update (event)
    timerStart ("mqtt.timer", 60000);

    local speed = tagGetNum ("local.wind_speed");
    local direction = tagGetNum ("local.wind_dir");
    local temperature = tagGetNum ("local.temperature");
    local humidity = tagGetNum ("local.humidity");
    local dewpoint = tagGetNum ("local.dewpoint");
    local pressure = tagGetNum ("local.pressure");

    print ("speed is " .. speed ..
           ", dir is " .. direction);

    -- TODO some local preprocessing required but for now we are just
    --      going to report raw values

--[[    speed_values[count] = speed;
    direction_values[count] = direction;

    count = count + 1;

    print ("store done");

    if (count > max) then
        max = count;
    end

    if (count == 10) then
        count = 0;
    end

    i = 0;

    local avg_speed = 0;
    local avg_dir = 0;

    print ("Calc avg over " .. max .. "values");

    while (i < max) do
        avg_speed = avg_speed + speed_values[i];
        avg_dir = avg_dir + direction_values[i];
        i = i + 1
    end

    avg_speed = 20*avg_speed / max;
    avg_dir = avg_dir * (360 / 16) / max;
]]

    mqttPublish(mqtt_client,topic_prefix .. "wind_speed",tostring(speed),0,false);
    mqttPublish(mqtt_client,topic_prefix .. "wind_dir",tostring(direction),0,false);
    mqttPublish(mqtt_client,topic_prefix .. "temperature",tostring(temperature),0,false);
    mqttPublish(mqtt_client,topic_prefix .. "humidity",tostring(humidity),0,false);
    mqttPublish(mqtt_client,topic_prefix .. "dewpoint",tostring(dewpoint),0,false);
    mqttPublish(mqtt_client,topic_prefix .. "pressure",tostring(pressure),0,false);
    print ("done");
end

function sensor_activate ()
    if (sensor_is_active ~= 0) then
        print "sensor already active"
        return
    end

    sensor_is_active = 1

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=wind_speed," ..
                      "name=Current Wind Speed," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=knots,scale=1,mobile=n," ..
                      "topic=" .. topic_prefix .. "wind_speed," ..
                      location)

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=wind_dir," ..
                      "name=Current Wind Directon," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=degrees,scale=22.5,mobile=n," ..
                      "topic=" .. topic_prefix .. "wind_dir," ..
                      location)

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=temperature," ..
                      "name=Current Temperature," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=degrees,scale=1,mobile=n," ..
                      "topic=" .. topic_prefix .. "temperature," ..
                      location)

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=humidity," ..
                      "name=Current Humidity," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=percent,scale=1,mobile=n," ..
                      "topic=" .. topic_prefix .. "wind_dir," ..
                      location)

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=dewpoint," ..
                      "name=Current Dewpoint," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=degrees,scale=1,mobile=n," ..
                      "topic=" .. topic_prefix .. "dewpoint," ..
                      location)

    mqttPublish (mqtt_client, server_topic,
                      "cmd=newsensor," ..
                      "id=pressure," ..
                      "name=Current Atmospheric Pressure," ..
                      "hwaddr=TDB," ..
                      "type=environmental,units=hPa,scale=1,mobile=n," ..
                      "topic=" .. topic_prefix .. "pressure," ..
                      location)

    timerStart ("mqtt.timer", 1000);
    eventHandler ("mqtt.timer", data_update);
end

function command_handler (event)
    data = tagGetString (event)

    command = nameValue (data, "cmd")

    print ("CMD recv " .. command)
    if (command == "regack") then
        print "regack received"
        sensor_activate ()
    else
        print ("*UNKNOWN* data " .. data .. " from device")
    end
end

local announce_topic="announce_weather"

-- Send an announce message to the server and register and event handler
-- for any received commands

function register (mqtt_client)
    mqttSubscribe (mqtt_client, device_topic)

    tagSetEvent(device_topic, "")
    eventHandler (device_topic, command_handler)

    print "publishing announce"
    mqttPublish(mqtt_client,announce_topic,
                     "vendor=emutex,address=" .. hwaddr .. "," ..
                     "name=Emutex Labs Weather Station")

end

function isLive(event)
    timerStart ("anounce.timer", 300000);
    register();
end

local mqtt_id = "lfc"
local mqtt_broker="cougar.emutexlabs.com"
local mqtt_port=1883
local mqtt_keepalive=5
local mqtt_client=0

function main (void)
    libOpen ("mqtt.so.1.0");
    print ("doing mqtt setup");
    mqtt_client=mqttNew(true);
    print ("doing mqtt up set");
    mqttUsername_pw_set(mqtt_client, "mqtt", "mqtt");
    print ("doing mqtt bind");

    -- TODO check return value from connect

    mqttStart(mqtt_client,mqtt_broker,mqtt_port,mqtt_keepalive);

    register(mqtt_client)

    timerStart ("anounce.timer", 300000);
    tagSetEvent ("anounce.timer", "update");
    eventHandler ("anounce.timer", isLive);

    print ("entering dispatch loop");
    while 1 do
        local event = eventWait ();
        eventDispatch (event);
    end
end

main ()
