-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local deviceControl = require 'utils/device_control'
local query = require 'db/query'
local tags = require 'utils/tags'
local config = require 'utils/config'
local utils = require 'utils/utils'
local usensor = require 'utils/sensor'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local mqttClient = params[3]

local actuatorThreads = {}
local actuatorSubscribedTopics = {}
local RECONFIG = tags.getActuatorsReconfig ()

local function startActuators ()
    NOTIFY ('Starting actuators...')

    local queryObj = query.
        SELECT ('sensor_ID, type, hwtype, hwaddr'):
        FROM ('sensor'):
        WHERE ('vendor_ID', '=', vendorID):
        AND ('device_ID', '=', deviceID):
        AND ('active', '=', 1)

    local count = 0
    queryObj:prepare()
    for row in query.iterator(queryObj) do
        local hwtype = usensor.legacyHwType2HwType(row.hwtype, row.type)

        if usensor.isActuator(hwtype) then
            count = count + 1
            local sensorID = row.sensor_ID

            NOTIFYF ('Subscribing to %s', sensorID)
            mqttSubscribeLocal (mqttClient, sensorID)
            actuatorSubscribedTopics[sensorID] = true

            if usensor.hasThread(hwtype) then
                NOTIFYF ('Starting thread %s.%s.%s', vendorID, deviceID, sensorID)

                local devFile = string.format ('devices/%s', hwtype)
                local hwaddrTable = paramDecode (row.hwaddr, config.HWADDR_PARSER)
                hwaddrTable = utils.toArray (hwaddrTable) or {row.hwaddr}

                thread (devFile, vendorID, deviceID, sensorID, unpack (hwaddrTable))
                actuatorThreads[sensorID] = true
            end
        end
    end
    queryObj:finish()
    NOTIFYF ('Configured %d actuators', count)
end

local function stopActuators ()
    NOTIFY ('Stopping actuators...')
    for sensorID,_ in pairs (actuatorThreads) do
        deviceControl.stopDeviceThread (vendorID, deviceID, sensorID)
    end

    for sensorID,_ in pairs (actuatorSubscribedTopics) do
        NOTIFYF ('Unsubscribing from %s', sensorID)
        mqttUnsubscribeLocal (mqttClient, sensorID)
    end

    -- clear threads list
    actuatorThreads = {}
    actuatorSubscribedTopics = {}
end

local function config ()
    stopActuators ()
    startActuators ()
end

-- do an initial config
config ()
eventHandler (RECONFIG, function () end)
while true do
    local event, param = eventWait ()
    if event == RECONFIG then
        -- config () uses eventWait () which calls a yield
        -- yielding is not possible in event handlers
        -- so config () has to be executed outside an event handler
        config ()
    else
        eventDispatch (event, param)
    end
end
