-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local tags = require 'utils/tags'
local config = require 'utils/config'
local alarm = require 'utils/alarm'

local params = {...}
local mqttClient = params[1]
local vendorID = params[2]
local deviceID = params[3]
local alarmState = {}
local parserType = config.getParserType (vendorID, deviceID)
local CLIENT_ALARM = tags.getClientAlarm(vendorID, deviceID)
local HANDSHAKE_DONE = tags.getHandshakeDone (vendorID, deviceID)

alarm.crashFile      = "/var/run/ubiworx/crash"
alarm.backtraceFile  = "/var/run/ubiworx/backtrace"

function alarm.shouldTriggerCrashAlarm(vendorID, deviceID)
    local file = io.open(alarm.crashFile, "r")

    if not file then
        return
    end
    file:close()

    local notes = {}
    for line in io.lines(alarm.crashFile) do
        table.insert(notes, line)
    end
    os.remove(alarm.crashFile)

    file = io.open(alarm.backtraceFile, "r")
    if file then
        file:close()

        for line in io.lines(alarm.backtraceFile) do
            table.insert(notes, line)
        end
        os.remove(alarm.backtraceFile)
    end

    local alarmTable = {
        alarm_ID = alarm.buildAlarmId({"ubiworx", "crash", deviceID}),
        vendor_ID = vendorID,
        device_ID = deviceID,
        sensor_ID = "",
        seconds = tonumber(os.time()),
        value = 0,
        category = alarm.systemCategory,
        latitude = 0,
        longitude = 0,
        notes = table.concat(notes, "<br>"),
        name = string.format("Gateway %s has unexpectedly stopped", deviceID),
        active = 1,
    }

    local msg = paramEncode(alarmTable, alarm.internalParser)
    tagSetString (tags.alarm, msg)
    alarmTable.active = 0;
    msg = paramEncode(alarmTable, alarm.internalParser)
    tagSetString (tags.alarm, msg)
end

local function alarmHandler (tag ,value)
    DEBUG ('Handling alarm "' .. value .. '"')
    local alarmTable = paramDecode (value, alarm.internalParser)

    if not alarmTable then
        WARNF ('Received unparseable alarm %q', value)
        return
    end

    alarmTable.active = tonumber(alarmTable.active)
    alarmTable.vendor_ID = vendorID
    alarmTable.device_ID = deviceID

    local ret, err = alarm.verifyMsg (alarmTable, {})
    if not ret then
        ERRORF ('Generated invalid alarm %q: %s', value, err)
        return
    end

    local alarmIndex = tags.buildUbiworxTag({alarmTable.alarm_ID, alarmTable.vendor_ID, alarmTable.device_ID})
    if alarmState[alarmIndex] ~= alarmTable.active and alarmTable.active == 1 then
        NOTIFYF ('Activating alarm %q: %q', alarmTable.name, alarmTable.notes)
    elseif alarmState[alarmIndex] == 1 and alarmTable.active == 0 then
        NOTIFYF ('Deactivating alarm %q: %q', alarmTable.name, alarmTable.notes)
    else
        DEBUG ('Ignored alarm "' .. value .. '"')
        return
    end
    alarmState[alarmIndex] = alarmTable.active

    local alarmTag = tags.alarm
    if tagGetNum(HANDSHAKE_DONE) > 0 then
        alarmTag = CLIENT_ALARM
    end

    mqttPublish (mqttClient, alarmTag, paramEncodeHeader (alarmTable, parserType))

    return true
end

-- MAIN
eventHandler (tags.alarm, alarmHandler)

-- Check crash report, if found trigger crash alarm
alarm.shouldTriggerCrashAlarm(vendorID, deviceID)

NOTIFY ('Alarm thread started')

while true do
    local event, param = eventWait ()
    eventDispatch (event, param)
end
