-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local config = require 'utils/config'
local tags = require 'utils/tags'
local json = require 'third_party/dkjson'

local statusFile = '/var/run/ubiworx/upgrade_status'
local upgradeFile = '/var/run/ubiworx/upgrade'

local installedPackages = {}

local packages = {}

function packages.restartUbiworx ()
    os.execute ('cp /usr/share/ubiworx/shell/ubiworx-upgrade.sh /tmp')
    os.execute ('nohup /tmp/ubiworx-upgrade.sh &')
end

local function updateUpgradeFile(file, package, version)
    file:write(string.format('%s=%s\n', package, version))
end

function packages.versionCheck (vendorID, deviceID)
    local softwareRequestJson = config.getSoftwareUpgradeRequest(vendorID, deviceID)
    local softwareRequest = json.decode(softwareRequestJson)

    local upgrade = io.open (upgradeFile, 'w')
    local anyUpdates = false

    NOTIFY('Checking for software updates...')
    for _, requestedSoftware in pairs(softwareRequest) do
        local requestedPackage = requestedSoftware.package
        local requestedVersion = requestedSoftware.version
        local installedPackage = installedPackages[requestedPackage]

        if not installedPackage then
            NOTIFYF('%s not found. Adding to upgrade file', requestedPackage)
            updateUpgradeFile(upgrade, requestedPackage, requestedVersion)
            anyUpdates = true
        else
            if requestedVersion == installedPackage.installed_version then
                NOTIFYF('Package %q is up to date. Ignoring...', requestedPackage)

            elseif installedPackage.status == 'FAIL' and
                installedPackage.requested_version == requestedVersion then
                NOTIFYF('Package %q already failed to update to version %s, not retrying',
                    requestedPackage, requestedVersion)

            else
                NOTIFYF('Package %q needs to be upgraded from %s to %s', requestedPackage,
                    installedPackage.installed_version,
                    requestedVersion)

                updateUpgradeFile(upgrade, requestedPackage, requestedVersion)
                anyUpdates = true
            end
        end
    end

    upgrade:close()

    if anyUpdates then
        packages.restartUbiworx()
    end
end

function packages.init(vendorID, deviceID)

    local f = io.open(statusFile, 'r')
    if not f then
        NOTIFY ('Package upgrade status file not found')
        return
    end
    io.close(f)

    NOTIFYF('Reading software status file at %s', statusFile)
    for line in io.lines(statusFile) do
        if line:sub(1, 1) ~= '#' then
            local status, package, installedVersion, requestedVersion =
                line:match('([^ ]+)%s+(.*)%s+(.*)%s+(.*)')

            installedPackages[package] = {
                installed_version = installedVersion,
                requested_version = requestedVersion,
                status = status,
            }

            NOTIFYF ('Package: %q, status: %q, installed version: %q, requested version: %q',
                package, status, installedVersion, requestedVersion)
        end
    end

    local softwareUpgradeStatus = json.encode(installedPackages)
    tagSetString(tags.softwareUpgradeStatus, softwareUpgradeStatus)
end

return packages
