-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

libOpen ("mqtt.so.1.0")
libOpen ("json.so.1.0")

local config = require "utils/config"
local tags = require "utils/tags"

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local mqttClient = params[3]
local timerNumber = 0
local PARSER_TYPE

local function configRules ()
    local rules = config.getRules(vendorID, deviceID)

    if rules == "" then
        NOTIFY ("No rules found")
        return
    end

    INFO ("Clearing old rules")
    rulesClear (mqttClient)

    INFO ("Installing new rules")
    rulesInstall (vendorID, rules)
end

local function subscribeRemote (event, param)
    local paramTable = paramDecode (param, PARSER_TYPE)
    local topic = tags.getSensorTag (vendorID, paramTable.deviceID, paramTable.sensorID)
    NOTIFYF ("%s, subscribe to topic '%s'", event, topic)
    mqttSubscribe (mqttClient, topic)
end

local function publishRemote (event, param)
    local paramTable = paramDecode (param, PARSER_TYPE)
    local topic = tags.getRemoteTopic(vendorID, paramTable.topic)
    NOTIFYF ("%s, publish %s to topic '%s'", event, paramTable.value, topic)
    mqttPublish (mqttClient, topic, paramTable.value)
end

eventHandler(tags.getRulesReconfig (), configRules)

NOTIFY ("Set trap for rules-remote")
eventHandler(tags.getRulesRemotePublish (), publishRemote)
eventHandler(tags.getRulesRemoteSubscribe (), subscribeRemote)

-- do this AFTER setting up events
if (libOpen ("rules.so.1.0") ~= true) then
    return
end

PARSER_TYPE = rulesParserType ()
configRules ()

while true do
    local event, param = eventWait()
    eventDispatch(event, param)
end
