-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local DAO = {}

local bool2num = {
    [false] = 0,
    [true] = 1,
}

function DAO.new(table)
    local dao = {}
    setmetatable(dao, { __index = DAO })
    dao._table = table
    dao._dirtyfields = {}
    return dao
end

function DAO:set(field, expectedType, value)
    assert(type(value) == expectedType,
        string.format('Wrong type for field %s, expected %q, got %q',
        field, expectedType, type(value)))

    if self._table[field] ~= value then
        self._table[field] = value
        self._dirtyfields[field] = true
    end
end

function DAO:dirtyFields()
    local dirties = {}
    for k, _ in pairs(self._dirtyfields) do
        dirties[k] = self._table[k]
    end
    return dirties
end

function DAO:clearDirtyFields()
    self._dirtyfields = {}
end

function DAO:isDirty()
    return next(self._dirtyfields)
end

-- shared getters
function DAO:isRemoved()
    return self._table.removed == 1
end

function DAO:isActive()
    return self._table.active == 1
end

function DAO:isMobile()
    return self._table.mobile == 1
end

function DAO:getName()
    return self._table.name
end

function DAO:getMobile()
    return self._table.mobile
end

function DAO:getLongitude()
    return self._table.longitude
end

function DAO:getLatitude()
    return self._table.latitude
end

-- shared setters
function DAO:setActive(v)
    self:set('active', 'number', bool2num[v])
end

function DAO:setRemoved(v)
    self:set('removed', 'number', bool2num[v])
end

function DAO:setLatitude(v)
    self:set('latitude', 'number', v)
end

function DAO:setLongitude(v)
    self:set('longitude', 'number', v)
end

function DAO:setName(v)
    self:set('name', 'string', v)
end

return DAO
