-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local query = require 'db/query'
local DAO = require 'db/dao'
local unode = require 'utils/node'

local Node = {}
local NodeMetaClass = { __index = Node }
setmetatable(Node, { __index = DAO })

function Node.new(table)
    local node = DAO.new(table)
    setmetatable(node, NodeMetaClass)
    node._nodeid = table.node_ID
    return node
end

function Node:isServerDefined()
    return self._table.origin == 2
end

function Node:getType()
    return self._table.type
end

function Node:getHardwareAddress()
    return self._table.hwaddr
end

function Node:getHardwareType()
    return self._table.hwtype
end

function Node:getOrigin()
    return self._table.origin
end

function Node:getTimestamp()
    return self._table.timestamp
end

function Node:getSampleRate()
    return self._table.sampleRate
end

function Node:setType(v)
    self:set('type', 'number', v)
end

function Node:setOrigin(v)
    self:set('origin', 'number', v)
end

function Node:setMobile(v)
    self:set('mobile', 'number', v)
end

function Node:setTimestamp(v)
    self:set('timestamp', 'number', v)
end

function Node:setSampleRate(v)
    self:set('sampleRate', 'number', v)
end

function Node:setHardwareAddress(v)
    self:set('hwaddr', 'string', v)
end

function Node:setHardwareType(v)
    self:set('hwtype', 'string', v)
end

function Node:toInsertQuery()
    return query.INSERT_INTO('node'):VALUE_TABLE(self._table)
end

function Node:toUpdateQuery()
    if not self:isDirty() then
        return nil
    end

    local vendorid = self._table.vendor_ID
    local deviceid = self._table.device_ID
    return query.UPDATE('node'):
        SET(self:dirtyFields()):
        WHERE('vendor_ID', '=', vendorid):
        AND('device_ID', '=', deviceid):
        AND('node_ID', '=', self._nodeid)
end

return Node
