-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local query = require 'db/query'
local DAO = require 'db/dao'
local usensor = require 'utils/sensor'

local Sensor = {}
local SensorMetaClass = { __index = Sensor }
setmetatable(Sensor, { __index = DAO })

function Sensor.new(table)
    local sensor = DAO.new(table)
    setmetatable(sensor, SensorMetaClass)
    sensor._sensorid = table.sensor_ID
    sensor._nodeid = table.node_ID
    return sensor
end

function Sensor:isLegacy()
    local hwtype = usensor.legacyHwType2HwType(self._table.hwtype, self._table.type)
    return self._table.hwtype ~= hwtype
end

function Sensor:updateFromLegacy()
    local hwtype = usensor.legacyHwType2HwType(self._table.hwtype, self._table.type)
    self:set('hwtype', 'string', hwtype)
end

function Sensor:isSensor()
    local hwtype = usensor.legacyHwType2HwType(self._table.hwtype, self._table.type)
    return usensor.isSensor(hwtype)
end

function Sensor:isActuator()
    local hwtype = usensor.legacyHwType2HwType(self._table.hwtype, self._table.type)
    return usensor.isActuator(hwtype)
end

function Sensor:isServerDefined()
    return self._table.origin == 2
end

function Sensor:getType()
    return self._table.type
end

function Sensor:getHardwareAddress()
    return self._table.hwaddr
end

function Sensor:getHardwareType()
    return self._table.hwtype
end

function Sensor:getMin()
    return self._table.min
end

function Sensor:getMax()
    return self._table.max
end

function Sensor:getOffset()
    return self._table.offset
end

function Sensor:getOrigin()
    return self._table.origin
end

function Sensor:getTimestamp()
    return self._table.timestamp
end

function Sensor:getScale()
    return self._table.scale
end

function Sensor:getUnits()
    return self._table.units
end

function Sensor:getSampleRate()
    return self._table.sampleRate
end

function Sensor:setType(v)
    self:set('type', 'number', v)
end

function Sensor:setOffset(v)
    self:set('offset', 'number', v)
end

function Sensor:setMax(v)
    self:set('max', 'number', v)
end

function Sensor:setMin(v)
    self:set('min', 'number', v)
end

function Sensor:setOrigin(v)
    self:set('origin', 'number', v)
end

function Sensor:setMobile(v)
    self:set('mobile', 'number', v)
end

function Sensor:setTimestamp(v)
    self:set('timestamp', 'number', v)
end

function Sensor:setSampleRate(v)
    self:set('sampleRate', 'number', v)
end

function Sensor:setScale(v)
    self:set('scale', 'number', v)
end

function Sensor:setHardwareAddress(v)
    self:set('hwaddr', 'string', v)
end

function Sensor:setHardwareType(v)
    self:set('hwtype', 'string', v)
end

function Sensor:setUnits(v)
    self:set('units', 'string', v)
end

function Sensor:toInsertQuery()
    return query.INSERT_INTO('sensor'):VALUE_TABLE(self._table)
end

function Sensor:toUpdateQuery()
    if not self:isDirty() then
        return nil
    end

    local vendorid = self._table.vendor_ID
    local deviceid = self._table.device_ID
    return query.UPDATE('sensor'):
        SET(self:dirtyFields()):
        WHERE('vendor_ID', '=', vendorid):
        AND('device_ID', '=', deviceid):
        AND('sensor_ID', '=', self._sensorid)
end

return Sensor
