-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: BMF136
   Description: Monitors up to 12 three phase circuits or 36 single phase circuits, or any combination of single or three phase circuits.
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1: STR "Serial port"
   Param2: STR "Address"
--]]

local Bfm136 = require 'drivers/bfm136'
local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]
local addr = params[5]
local bfmObj = Bfm136.create (serport, addr)

local definedTags = {}
local definedRanges = {}

local function isRangeDefined (submeter, startAddr, endAddr)
    definedRanges[submeter] = definedRanges[submeter] or {}

    for _, range in ipairs (definedRanges[submeter]) do
        if range.startAddr == startAddr and
           range.endAddr == endAddr then
           return true
        end
    end
    return false
end

local function defineRegister (tag, value)

    local paramTable = paramDecode (value, 'csv')

    local submeter = tonumber (paramTable.submeter)
    local startAddr = tonumber (paramTable.startAddr)
    local endAddr = tonumber (paramTable.endAddr)
    local register = tonumber (paramTable.register)

    if not submeter or not startAddr or not endAddr or not register or
        not paramTable.tag then
        ERRORF ('BFM 136 Failed to define a register, invalid values %q', value)
        return
    end

    if definedTags[paramTable.tag] then
        return
    end

    if not isRangeDefined (submeter, startAddr, endAddr) then
        bfmObj:defineRange (submeter, startAddr, endAddr)
        local range = {
            startAddr = startAddr,
            endAddr = endAddr
        }
        table.insert (definedRanges[submeter], range)
    end

    bfmObj:defineRegister (submeter, register, paramTable.tag)
    definedTags[paramTable.tag] = true
end

if not bfmObj then
    return
end

eventHandler ('_bfm136register', defineRegister)

deviceControl.loopSensorThread (vendorID, deviceID, sensorID, function () end,
    3600000)

eventHandlerClear ('_bfm136register')
bfmObj:destroy ()
