-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: BME280
   Description: Temperature, humidity and pressure sensor
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1: STR "Bus"
   Param2: STR "Mode"    "WEATHER"
   Param3: STR "Address" "0x77"
--]]

libOpen ('i2c.so.1.0')
libOpen ('bme280.so.1.0')

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local MODES = {
    HUMIDITY = 0,
    WEATHER = 1,
    INDOOR = 2
}

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local bus = tonumber (params[4])
local mode = MODES[params[5]]

-- Device address of the sensor is 0x77 but can be changed to 0x76
local addr = 0x77
if params[6] then
    addr = tonumber (params[6])
end

if not bus or not mode then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function updateBme280 ()
    local measure = bme280ReadComp ()

    if not measure then
        sensorSet (sensorID .. '-temp', nil)
        sensorSet (sensorID .. '-press', nil)
        sensorSet (sensorID .. '-hum', nil)
    else
        sensorSet (sensorID .. '-temp', measure.temp)
        sensorSet (sensorID .. '-press', measure.press)
        sensorSet (sensorID .. '-hum', measure.hum)
    end
end

if not bme280Open (bus, addr, mode) then
    ERRORF ('Failed to open BME 280 at 0x%02x on bus %d', addr, bus)
    bme280Close ()
    return
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, updateBme280, 1000)

bme280Close ()
