-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Carlo Gavazzi em21
   Description: Three-phase energy meter.
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1: STR "Serial port"
   Param2: STR "Address"
--]]

--- brief Script for managing a Carlo Gavazzi EM21 modbus device
-- See also:
-- http://www.sontay.com/sites/default/files/productdownloads/PM-EM21_Modbus.pdf

local modbusMgr = require 'drivers/modbus_manager'
local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]
local address = params[5]

if not serport or not address then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local dataFormat = 'FIELDBUS_DATA_FMT_INT32_CDAB' -- 32 bit value, BE byte order, LE double-word order
local interval = 5000  -- scan every 5 seconds
local noScale = 1
local scaleWatts = 10
local scaleVoltage = 10
local scaleCurrent = 100

local bus = modbusMgr.open (serport)
if bus < 0 then
    ERRORF ('%s failed to open modbus for %q', utils.getScriptName (), sensorID)
    return
end

local dev = modbusRegisterDevice (bus, address)
if dev < 0 then
    ERRORF ('%s failed to register device for %q', utils.getScriptName (), sensorID)
    modbusMgr.close (bus)
    return
end

-- Define the register range for voltage readings
modbusDefineInputRegisterRange (dev,
				  0x00, 0x05,
				  dataFormat, scaleVoltage, interval)

-- Define the register range for current readings
modbusDefineInputRegisterRange (dev,
				  0x0c, 0x11,
				  dataFormat, scaleCurrent, interval)

-- Define the register range for watts
modbusDefineInputRegisterRange (dev,
				  0x28, 0x29,
				  dataFormat, scaleWatts, interval)

modbusDefineInputRegister (dev, 0x00, sensorID .. '-v-l1')
modbusDefineInputRegister (dev, 0x02, sensorID .. '-v-l2')
modbusDefineInputRegister (dev, 0x04, sensorID .. '-v-l3')
modbusDefineInputRegister (dev, 0x0c, sensorID .. '-i-l1')
modbusDefineInputRegister (dev, 0x0e, sensorID .. '-i-l2')
modbusDefineInputRegister (dev, 0x10, sensorID .. '-i-l3')
modbusDefineInputRegister (dev, 0x28, sensorID .. '-watts')

deviceControl.loopDeviceThread(vendorID, deviceID, sensorID)

modbusUnregisterDevice (dev)
modbusMgr.close (bus)
