-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: CPU free
   Description: Return the current amout of free CPU
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Unit: Percent
   Min: 0
   Max: 100
--]]

--! @file.
--! @brief Script to periodically calculate free CPU

local utils = require 'utils/utils'
local deviceControl = require 'utils/device_control'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]

local lastTotal = 0
local lastIdle = 0

local function updateCpufree ()
    local fsocket= io.popen('cat /proc/stat');
    if not fsocket then
        WARN ('Failed to open cpufree file "/proc/stat"')
        return
    end
    local t= fsocket:read('*all');
    fsocket:close();
    local table = utils.split(t)
    local user = table[2]
    local nice = table[3]
    local system = table[4]
    local idle = table[5]
    local io = table[6]
    local irq = table[7]
    local softirq = table[8]
    local total = user+nice+system+idle+io+irq+softirq
    local deltaTotal = total - lastTotal
    local deltaIdle = idle - lastIdle
    lastTotal = total
    lastIdle = idle
    if (deltaTotal == 0) then deltaTotal = 1 end
    local free = deltaIdle * 100 / deltaTotal
    return free
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, updateCpufree, 1000)
