-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: MCP4725 I2C DAC
   Description: MCP4725 I2C Digital to analog converter
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Generic
   Param1: STR "Bus"
   Param2: STR "Address" "0x60"
--]]

local i2c = require 'drivers/i2c'
local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local bus = tonumber (params[4])
local addr = tonumber (params[5] or 0x60)
local REG = 0x40

local MAX = 0xFFF
local MIN = 0x000

if not bus or not addr then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local dac = i2c.create (bus, addr)
if not dac then
    ERROR ('Failed to open I2C bus for DAC')
    return
end

local function updateDac (tag, value)
    local num = tonumber (value)
    if not num then
        WARNF ('Got invalid value for DAC %s', tostring (value))
        return
    end

    if num < 0 then
        num = 0
    end
    if num > 100 then
        num = 100
    end

    num = math.floor (num * MAX / 100)
    INFO("Setting DAC to " .. num)

    -- MCP4725 needs a left-justified 12-bit value sent in 2 bytes (big-endian)
    local bytearray = string.char(bit32.lshift(num, -4),
                                  bit32.lshift(bit32.band(num, 0xF), 4))
    dac:writeRaw (REG, bytearray)
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, updateDac)
