-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: DISK free
   Description: Return the current amout of free DISK
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Unit: Percent
   Min: 0
   Max: 100
   Param1: STR "Partition"
--]]

--! @file.
--! @brief Script to periodically calculate free DISK

local utils = require 'utils/utils'
local deviceControl = require 'utils/device_control'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local partition = params[4]

-- If match_fs is set, this will run df with no path and attempt
-- to match the filesystems and mount-points listed in the output
-- Otherwise, the user-specified path is passed to df.  This allows
-- the user to pass paths for specific directories (e.g /var/log/)
-- without needing to know exactly which filesystem or mount-point
-- the path is located on.
local function getDiskfree (partition, match_fs)
    local cmd = 'df -P '
    if not match_fs then
        cmd = cmd .. partition .. ' 2>&1'
    end
    local f = io.popen(cmd);
    if not f then
        WARN ('Failed to execute disk free')
        return
    end

    local free

    for line in f:lines() do
        local fs, blocks, used, avail, use, mount
            = line:match('^(%g+)%s+(%d+)%s+(%d+)%s+(%d+)%s+([%d]+)%%%s+(%g+)$')

        if mount and ((not match_fs) or (fs == partition or mount == partition)) then
            INFO ("parsed "..blocks.." and "..avail.." for "..partition)
            free = 100 * tonumber(avail) / tonumber(blocks)
            break
        end
    end
    f:close();

    return free
end

local function updateDiskfree ()
    local free

    -- Try passing the path to df first, and fall back to searching df output
    free = getDiskfree(partition, false) or getDiskfree(partition, true)

    if free == nil then
        WARN ("path "..partition.." not found for disk free report")
        return 0
    end

    return free
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, updateDiskfree, 1000)
