-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: SM6 Geophone
   Description: SM6 Omni-directional Geophone over I2C ADC Pi Plus
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1: INT "Bus"
   Param2: INT "Hat1 low address"
   Param3: INT "Hat1 hig address"
   Param4: INT "Hat2 low address"
   Param5: INT "Hat2 hig address"
   Param6: INT "X channel"
   Param7: INT "Y channel"
   Param8: INT "Z channel"
   Param9: INT "FFT window"
   Param10: INT "Sample rate"
   Param11: INT "v2mms"
--]]

libOpen('geophone-sm6.so.1.0')

local deviceControl = require 'utils/device_control'

local params = { ... }
local vendorID      = params[1]
local deviceID      = params[2]
local sensorID      = params[3]
local bus           = tonumber(params[4])
local hat1_lowaddr  = tonumber(params[5])
local hat1_highaddr = tonumber(params[6])
local hat2_lowaddr  = tonumber(params[7])
local hat2_highaddr = tonumber(params[8])
local xchannel      = tonumber(params[9])
local ychannel      = tonumber(params[10])
local zchannel      = tonumber(params[11])
local fftWindow     = tonumber(params[12])
local sampleRate    = tonumber(params[13])
local v2mms         = tonumber(params[14])

local bitrate = 12
local pga = 1
local mode = 1

local config = {
    id = sensorID,
    i2cbus = bus,
    fftwindow = fftWindow,
    sampleRate = sampleRate,
    v2mms = v2mms,
    x = {
        addr = hat1_lowaddr,
        channel = xchannel,
        bitrate = bitrate,
        pga = pga,
        mode = mode,
    },
    y = {
        addr = hat1_highaddr,
        channel = ychannel,
        bitrate = bitrate,
        pga = pga,
        mode = mode,
    },
    z = {
        addr = hat2_lowaddr,
        channel = zchannel,
        bitrate = bitrate,
        pga = pga,
        mode = mode,
    },
}

if not geophoneInit(config) then
    return nil
end

deviceControl.loopDeviceThread(vendorID, deviceID, sensorID)

geophoneClose()
