-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: GPO (Edison)
   Description: General Purpose Output for Edison
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Edison
   Max: 1
   Min: 0
   Param1: STR "Gpio number"
--]]

local deviceControl = require 'utils/device_control'
local muxing = require 'drivers/edison/muxing'
local utils = require 'utils/utils'
local query = require 'db/query'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local pinNum = params[4]

if not pinNum then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local gpioNumber = muxing.getGpioNumber(pinNum)
local gpioPath = string.format('/sys/class/gpio/gpio%s', gpioNumber)

local function readCallback()
    local f = io.open(gpioPath .. '/value', 'r');
    if f == nil then
        ERRORF ('Failed to read gpio %q', gpioNumber)
        return
    end
    local valueStr = f:read('*all')
    f:close()

    local value = tonumber(valueStr)
    if value == nil then
        ERRORF ('Gpio %q read invalid output %q', gpioNumber, valueStr)
        return
    end
    return value
end

local function writeCallback(tag, value)
    value = tonumber(value) or 0

    local f = io.open(gpioPath .. '/value', 'w');
    if f == nil then
        ERROR('Failed to write gpio %q', gpioNumber)
        return
    end
    f:write(value)
    f:close()
end

local sensorRow = query.SELECT():
    FROM ('sensor'):
    WHERE ('vendor_ID', '=', vendorID):
    AND ('device_ID', '=', deviceID):
    AND ('sensor_ID', '=', sensorID):
    execute ().data[1]

if not muxing.muxGPIO(pinNum, 'out') then
    ERRORF('Failed to mux GPIO %q with direction %q',pinNum, gpioType)
    return
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, writeCallback)
