-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: IIO ADC PiPlus
   Description: Read ADC values over I2C from a Pi Plus Hat
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Param1: INT "Bus"
   Param2: INT "Hat low address"
   Param3: INT "Hat hig address"
   Param4: INT "Channel"
   Param5: INT "Sample rate" "1"
--]]

libOpen('piplus.so.1.0')

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID      = params[1]
local deviceID      = params[2]
local sensorID      = params[3]
local bus           = tonumber(params[4])
local hat_lowaddr   = tonumber(params[5])
local hat_highaddr  = tonumber(params[6])
local channel       = tonumber(params[7])
local sampleRate    = tonumber(params[8]) or 1

local bitrate = 12
local pga     = 1
local mode    = 1

local adcHandle = piplusNew(bus, hat_lowaddr, hat_highaddr, bitrate, pga, mode,
                            sampleRate, 1, 1, 1, 1, 1, 1, 1, 1)

if adcHandle < 0 then
    ERRORF ('Failed to initialize ADC on bus %q', bus)
    return
end

local function readAdc ()
  return piplusRead(adcHandle, channel)
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, readAdc, (499/sampleRate))

piplusClose(adcHandle)
