-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: LCD RGB UP
   Description: Drive an LCD with RGB backlight from Up Board
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   DataType: STR
   Platform: UpBoard
   Param1: STR "Bus"
   Param2: STR "Mode" "TEXT"
--]]

local LcdRgb = require 'drivers/up/lcd_rgb'
local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local bus = params[4]
local mode = params[5] or 'TEXT'
local lcdObj

if not bus then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function updateText(_, param)
    lcdObj:setText (param)
end

local function updateRGB(_, param)
    local col = paramDecode (param, 'csv')

    if not col then
        WARNF('LcdRgb unparsable color param %q', param)
        return
    end

    col.r = tonumber (col.r)
    col.g = tonumber (col.g)
    col.b = tonumber (col.b)

    if not col.r or not col.g or not col.b then
        WARNF('LcdRgb reveived invalid color param %q', param)
        return
    end
    lcdObj:setRGB (col.r, col.g, col.b)
end

lcdObj = LcdRgb.create(bus)
if not lcdObj then
    ERRORF ('Failed to create LcdRgb object for %q', sensorID)
    return
end

local MODES = {
    TEXT = updateText,
    RGB = updateRGB
}

if not MODES[mode] then
    ERRORF ('LcdRgb invalid mode %q', mode)
    return
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, MODES[mode])
