-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: LED Supermicro
   Description: Drive Supermicro LED
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Generic
   Param1: STR "LED id"
--]]

libOpen('bin_ops.so.1.0')

local i2c = require "drivers/i2c"
local deviceControl = require "utils/device_control"
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local ledid = params[4]

if not ledid then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function led_supermicro(id)

    -- There's only 8 LED outputs
    id = tonumber(id)
    assert(id >= 0 and id <= 7)

    -- The LP3944 chip can be found at I2C address 0x63 in Supermicro
    local self = {}
    self.i2c = i2c.create(0, 0x63)
    self.id = id

    local __LS0 = 0x06
    local __LS1 = 0x07

    self.led_id = id
    self.ls_reg = __LS0
    if self.led_id > 3 then
        self.led_id = self.led_id - 4
        self.ls_reg = __LS1
    end

    -- NOTE: LED's being driven in the opposite way since the circuit where
    -- it's  connected does not have a relay.
    self.led_off = binLeft(0x1, self.led_id * 2)
    self.led_on = binNeg(binLeft(0x3, self.led_id * 2))

    function self.set_output(output)
        local current_ls_reg = self.i2c:readByte(self.ls_reg)

        local led_val
        if  tonumber(output) > 0 then -- turn LED ON
            led_val = binAnd(current_ls_reg, self.led_on)
        else -- turn LED OFF
            led_val = binOr(current_ls_reg, self.led_off)
        end
        self.i2c:writeByte(self.ls_reg, led_val)
    end

    return self
end

-- Create LED handler for the given LED ID
local led = led_supermicro(ledid)

local function led_cb(tag, value)
    value = tonumber(value) or 0
    led.set_output(value)
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, led_cb)
