-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Network failover
   Description: Sets up different network metrics to ensure traffic goes through a main interface
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: OnDemandSensor
   Platform: Generic
   Param1: STR "Main interface"
   Param2: STR "Back up interface"
--]]

local deviceControl = require 'utils/device_control'
local config = require 'utils/config'
local tags = require 'utils/tags'

local params = { ... }
local vendorID        = params[1]
local deviceID        = params[2]
local sensorID        = params[3]
local mainInterface   = params[4]
local backupInterface = params[5]
local HANDSHAKE_DONE = tags.getHandshakeDone (vendorID, deviceID)
local broker = config.getClientBroker(vendorID, deviceID)
local handshakeShutdown = false

local function addRoute(host, iface, metric)
    local del = string.format("route del -host %s metric %d %s >/dev/null 2>&1", host, metric, iface)
    local add = string.format("route add -host %s metric %d %s >/dev/null 2>&1", host, metric, iface)

    NOTIFY('Running ' .. del)
    os.execute(del)
    NOTIFY('Running ' .. add)
    os.execute(add)
end

local function ping()
    NOTIFYF('Pinging %s through iface %s', broker, mainInterface)
    local ping = io.popen(string.format("ping -q -c 5 %s -i 0.2 -I %s",
        broker, mainInterface))
    local output = ping:read('*a')
    ping:close()
    local packetLossPercentage = string.match(output, '(%d+)%%')

    if not packetLossPercentage or tonumber(packetLossPercentage) > 75 then
        WARNF('Ping packet loss percentage is %s%%', tostring(packetLossPercentage))
        tagSetNum(HANDSHAKE_DONE, -1)
        handshakeShutdown = true
    elseif handshakeShutdown then
        NOTIFYF('Main interface %s seems to be back up', mainInterface)
        addRoute(broker, mainInterface, 10)
        handshakeShutdown = false
        tagSetNum(HANDSHAKE_DONE, 0)
    end
end

if not os.execute(string.format('ifconfig %s >/dev/null 2>&1', mainInterface)) then
    ERRORF('Main interface %q does not exist', mainInterface)
    return
end

if not os.execute(string.format('ifconfig %s >/dev/null 2>&1', backupInterface)) then
    ERRORF('Backup interface %q does not exist', mainInterface)
    return
end

addRoute(broker, mainInterface, 10)
addRoute(broker, backupInterface, 100)

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, ping, 30000)

