-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: 1-Wire temperature
   Description: 1-Wire temperature sensor
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Unit: C
   Max: 125
   Min: -55
   Param1: STR "1-Wire family" "28"
   Param2: STR "1-Wire serial" "0000074d4578"
--]]

libOpen("onewire.so.1.0")

local deviceControl = require 'utils/device_control'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local owdevice = string.format("%s-%s", params[4], params[5])

local function updateTemperature ()
    local ok, output = oneWireRead(owdevice)

    if not ok then
        NOTIFYF("No data avalible for one wire temperature sensor %s", owdevice)
        return nil
    end

    local n = 1
    for line in output:gmatch("[^\r\n]+") do
        if n == 1 then
            local ok1, _, valid = string.find(line, "^[ %d%a]+: crc=[%d%a]+ (%a+)$")
            if not ok1 or (valid ~= "YES") then
                WARN("Wrong CRC")
                return nil
            end
            n = 2
        elseif n == 2 then
            local ok2, _, value = string.find(line, "^[ %d%a]+t=(%d+)$")
            if not ok2 then
                WARN("Temperature value not found")
                return nil
            end
            return (tonumber(value) / 1000)
        end
    end

    return nil
end

oneWireNew(owdevice)

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, updateTemperature, 10000)

oneWireDelete(owdevice)
