-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Papouch TQS3 modbus
   Description: Mange Papouch TQS3 modbus device
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Param1: STR "Serial port"
   Param2: STR "Address"
--]]

--! \file
--! \brief Script for managing a Papouch TQS3 modbus device

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]
local address = params[5]

if not serport or not address then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

libOpen('libserial.so.1.0')
libOpen('fieldbus.so.1.0')
libOpen('modbus.so.1.0')

local dataFormat = 'FIELDBUS_DATA_FMT_INT16_AB' -- Big Endian short
local interval = 5000  -- scan every 5 seconds
local noScale = 1
local scaleDiv10 = 10

if modbusInstance == nil then
    INFO ('Opening ' .. serport)
    modbusInstance = modbusRtuOpen (serport)

    if (modbusInstance == -1) then
        WARN ('modbus open fail')
	return
    end

    modbusInstanceCount = 1
else
    modbusInstanceCount = modbusInstanceCount + 1
end

local dev = modbusRegisterDevice (modbusInstance, address);

if (dev == -1) then
    print ('register device fail')
    return
end

-- Define 6 holding registers from index 0 through 5 each one being
-- a 16-bit big endian value.

modbusDefineHoldingRegisterRange (dev,
                                    0x00, 0x05,
                                    dataFormat, noScale);

-- Define 2 input registers from index 0 through 1 each one being
-- a 16-bit big endian value.  Specify the scan interval as above.
-- Scaling factor is 10

modbusDefineInputRegisterRange (dev,
				    0x00, 0x00,
				    dataFormat, noScale, interval);

modbusDefineInputRegisterRange (dev,
				    0x01, 0x01,
				    dataFormat, scaleDiv10, interval);

-- Define tags for the holding registers

modbusDefineHoldingRegister (dev, 0, sensorID .. '.ConfigEnable');
modbusDefineHoldingRegister (dev, 1, sensorID .. '.Address');
modbusDefineHoldingRegister (dev, 2, sensorID .. '.BaudRate');
modbusDefineHoldingRegister (dev, 3, sensorID .. '.DataBits');
modbusDefineHoldingRegister (dev, 4, sensorID .. '.PktEndDelay');
modbusDefineHoldingRegister (dev, 5, sensorID .. '.Protocol');

-- Define tags for the input registers

modbusDefineInputRegister (dev, 0, sensorID .. 'TempStatus');
modbusDefineInputRegister (dev, 1, sensorID);

deviceControl.loopDeviceThread(vendorID, deviceID, sensorID)

modbusInstanceCount = modbusInstanceCount - 1
modbusUnregisterDevice (dev)

if (modbusInstanceCount == 0) then
    modbusClose (modbusInstance)
    modbusInstance = nil
end

