-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: PWM (Supermicro)
   Description: PWM for Supermicro
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Supermicro
   Param1: STR "PWM id"
--]]

libOpen('bin_ops.so.1.0')

local i2c = require 'drivers/i2c'
local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local pwmID = params[4]

if not pwmID then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function pwm_supermicro(id)

    -- There's only 8 LED outputs
    id = tonumber(id)
    assert(id >= 0 and id <= 7)

    -- The LP3944 chip can be found at I2C address 0x63 in Supermicro
    local self = {}
    self.i2c = i2c.create(0, 0x63)
    self.id = id

    self.FREQ_25HZ = 0x5

    -- The 8 LEDs are connected to 2 different PWM sources that can be
    -- controlled separately: DIM0 and DIM1. Use only DIM0 for simplicity.
    local __PSC0 = 0x02
    local __PWM0 = 0x03
    local __LS0 = 0x06
    local __LS1 = 0x07
    local __DIM0_SEL = 0x02

    function self.set_frequency(freq)
        self.i2c:writeByte(__PSC0, freq)
    end

    function self.set_duty(duty)
        self.i2c:writeByte(__PWM0, duty)
    end

    function self.enable()
        local led_id = self.id
        -- There's two LED selector registers at 0x06 and 0x07. LS0 controls
        -- LEDs 0-3 and LS1 LEDs 4-7. Depending on the LED id select the right
        -- LS register.
        local ls_reg = __LS0
        if led_id > 3 then
            led_id = led_id - 4
            ls_reg = __LS1
        end

        -- LS register has 2 bits per LED to control the type of output.
        -- A 0x02 value selects the DIM0 output.
        local dim0_sel = binLeft(__DIM0_SEL, led_id * 2)
        local current_ls_reg = self.i2c:readByte(ls_reg)
        dim0_sel = binOr(dim0_sel, current_ls_reg)
        self.i2c:writeByte(ls_reg, dim0_sel)
    end

    return self
end

-- Create PWM handler for the given LED ID
local pwm = pwm_supermicro(pwmID)

-- Set default values: frequency of 25Hz and 0% duty cycle
pwm.set_frequency(pwm.FREQ_25HZ)

-- Use a current value if any
pwm.set_duty(tagGetNum(sensorID))
pwm.enable()

local function pwm_cb(tag, value)
    value = tonumber(value) or 0
    pwm.set_duty(value)
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, pwm_cb)
