-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: RFID
   Description:
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: AsyncSensor
   Platform: Generic
   Param1: STR "Serial port"
   Param2: STR "Baudrate"
--]]

local utils = require 'utils/utils'
local deviceControl = require 'utils/device_control'
local Rfid = require 'drivers/rfid'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serialPort = params[4]
local baudrate = params[5]
local rfidObj

if not serialPort or not baudrate then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function updateRfid ()
    local id = rfidObj:read ()
    NOTIFYF ('Read RFID %q', id)
    return id
end

-- MAIN
rfidObj = Rfid.create (serialPort, baudrate)
if not rfidObj then
    ERRORF ('Failed to create RFID object')
    return
end

if not rfidObj:open () then
    ERRORF ('Failed to open serial connection for RFID %s %s',
        rfidObj:getPort (), rfidObj:getBaudrate ())
    return
end

deviceControl.loopSensorThread (vendorID, deviceID, sensorID, updateRfid, 100)

rfidObj:close ()
