-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: RFXCOM
   Description: Create an RF discovery sensor. This sensor listens for RF devices and sets a tag whenever any are found.
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1: STR "Serial port"
   Param2: STR "Should discover" "true"
--]]

--! \file
--! \brief Script to create an RF discovery sensor.  This sensor listens for RF
--!        devices and sets a tag whenever any are found.

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'
local query = require 'db/query'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]
local shouldDiscover = utils.tobool(params[5])

if not serport or shouldDiscover == nil then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

local function registerSensor (typ, param, id, index, units)
    local sensorID = string.format('%s-%s-%s', typ, param, id)
    local hwaddr = string.format('1=%s,2=%s,3=%s', id, index, param)
    local name = string.format('%s ID %s %s', typ, id, param)

    -- Query existing by hwtype and hwaddr, not by sensor id.  The user may
    -- change sensor id.
    local resultTable = query.
        SELECT():
        FROM('sensor'):
        WHERE('hwtype', '=', typ):
        AND('hwaddr', '=', hwaddr):
        execute()

    if resultTable.count > 0 then
        return
    end

    query.INSERT_INTO('sensor'):
        COLUMNS('vendor_ID', 'device_ID',
            'sensor_ID', 'name', 'type',
            'hwaddr', 'hwtype', 'scale',
            'offset', 'max', 'origin',
            'timestamp', 'min', 'units',
            'mobile', 'latitude',
            'longitude', 'active',
            'sampleRate'):
        VALUES(vendorID, deviceID, sensorID,
            name, 0, hwaddr, typ, 1,0,1000,1,
            0,0,'units',0,0,0,1,10000)
        :execute()
end

local function rfDiscover (tag, value)
    local typ = tonumber(nameValue(value, 'type'))
    local id = tonumber(nameValue(value, 'id'))

    if typ == 85 then
        registerSensor ('oregon-rain', 'battery', id, 0, '0-9')
        registerSensor ('oregon-rain', 'signal', id, 1, '0-9')
        registerSensor ('oregon-rain', 'rate', id, 2, 'mm per hr ???')
        registerSensor ('oregon-rain', 'total', id, 3, 'mm ???')
    elseif typ == 86 then
        registerSensor ('oregon-wind', 'battery', id, 0, '0-9')
        registerSensor ('oregon-wind', 'signal', id, 1, '0-9')
        registerSensor ('oregon-wind', 'gust', id, 2, 'm/s')
        registerSensor ('oregon-wind', 'direction', id, 3, '0-f')
        registerSensor ('oregon-wind', 'speed', id, 4, 'm/s')
    elseif typ == 82 then
        registerSensor ('thgr810', 'battery', id, 0, '0-9')
        registerSensor ('thgr810', 'signal', id, 1, '0-9')
        registerSensor ('thgr810', 'temp', id, 2, 'degrees C')
        registerSensor ('thgr810', 'hum', id, 3, 'percent')
        registerSensor ('thgr810', 'comfort', id, 4, '0 or 1')
    elseif typ == 16 then
        registerSensor('home-easy-pir1', 'battery', id, 0, '0-9')
        registerSensor('home-easy-pir1', 'signal', id, 1, '0-9')
        registerSensor('home-easy-pir1', 'command', id, 2, '0 or 1')
    elseif typ == 17 then
        registerSensor('home-easy-pir2', 'battery', id, 0, '0-9')
        registerSensor('home-easy-pir2', 'signal', id, 1, '0-9')
        registerSensor('home-easy-pir2', 'unit', id, 2, 'int')
        registerSensor('home-easy-pir2', 'command', id, 3, '0 or 1')
        registerSensor('home-easy-pir2', 'dim', id, 4, '???')
    end
end

if not rfxcomBus or rfxcomBus < 0 then
    NOTIFYF ('Opening port %s for access to rfxcom', serport)
    libOpen('libserial.so.1.0')
    libOpen('fieldbus.so.1.0')
    libOpen('rfxcom.so.1.0')
    rfxcomBus = rfxcomOpen (serport)
end

if rfxcomBus >= 0 and shouldDiscover then
    eventHandler ('rfx-discover', rfDiscover)
    rfxcomDiscover (rfxcomBus, 'rfx-discover')
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, function() end, 10000)

if rfxcomBus >= 0 then
    eventHandlerClear ('rfx-discover')
    rfxcomClose (rfxcomBus)
    rfxcomBus = nil
end

