-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Serial 16x2 LCD display
   Description: Writes to a serial 16x2 LCD display.
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Generic
   Param1: STR "Serial port"
--]]

--! @file.
--! @brief Script writes to a serial 16x2 LCD display.

libOpen('libserial.so.1.0')

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]

if not serport then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

NOTIFYF ('serLcd16x2 opening serial port %q', serport)
local display = serialOpen(serport, 9600)

local function updateLCD (tag, value)
    serialWrite(display, value, string.len(value))
end

if display == -1 then
    ERROR('Open serial failed')
    return
else
    --Set LCD size
    serialWrite(display, string.char(0xFE,0xD1,16,2), 4)
    --Autoscroll off
    serialWrite(display, string.char(0xFE,0x52), 2)
end

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, updateLCD)

serialClose(display)
