-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Serial temperature sensor
   Description: Reads temperature form the temperature serial sensor.
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Max: 50
   Min: -20
   Param1: STR "Serial port" "/dev/ttyUSB0"
--]]

--! @file.
--! @brief Script reads temperature form the temperature serial sensor.

libOpen('libserial.so.1.0')

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]

if not serport then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

NOTIFYF ('sertemp opening serial port %q', serport)
local tempSensor = serialOpen(serport, 9600)

local buffer = ''

local function updateSerTemp ()
    buffer = buffer .. serialRead(tempSensor, 1)

    -- Validate the read value is a valid temperature
    if buffer:match('^[+-]?%d+.%d+[CFcf]?%s*$') ~= nil then
        -- Remove letters and blanks and convert to number
        local temp = buffer:gsub('[%a%s]+', '')
        buffer = ''
        return tonumber(temp)
    end
end

if tempSensor == -1 then
    ERROR('Open serial failed')
    return
end

deviceControl.loopSensorThread(vendorID, deviceID, sensorID, updateSerTemp, 1000)

serialClose(tempSensor)
