-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: Sound level
   Description: Calculate sound level in dbA using a microphone
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Node
   Platform: Generic
   Param1:  STR "Sound device"   "default"
   Param2:  INT "Duration"       "60"
   Param3:  INT "Sample rate"    "44100"
   Param4:  INT "Channels"       "1"
   Param5:  INT "Max wav files"  "100"
   Param6:  STR "Wav files path" "/tmp"
   Param7:  STR "Mixer card"     ""
   Param8:  STR "Mixer mic"      ""
   Param9:  INT "Volume"         "100"
   Param10: DEC "Calibration"    ""
--]]

local deviceControl = require 'utils/device_control'

libOpen('sound.so.1.0')

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local soundDev = params[4]
local duration = params[5]
local sampleRate = params[6]
local channels = params[7]
local maxWavFiles = params[8]
local wavFilesPath = params[9]
local mixerCard = params[10]
local mixerMic = params[11]
local volume = params[12]
local calibration = params[13]

if not soundDev or
    not wavFilesPath or
    not mixerCard or
    not mixerMic or
    not tonumber(calibration) or
    not tonumber(duration) or
    not tonumber(sampleRate) or
    not tonumber(channels) or
    not tonumber(volume) or
    not tonumber(maxWavFiles) then

    ERRORF("Invalid configuration for sensor %s", sensorID)
    return
end

local config = {
    id = sensorID,
    soundDev = soundDev,
    duration = duration,
    sampleRate = sampleRate,
    channels = channels,
    maxWavFiles = maxWavFiles,
    wavFilesPath = wavFilesPath,
    mixerCard = mixerCard,
    mixerMic = mixerMic,
    volume = volume,
    calibration = calibration,
}

if not soundInit(config) then
    return
end

deviceControl.loopDeviceThread(vendorID, deviceID, sensorID)

soundClose()
