-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: SunGrow Inverter (ModBus TCP)
   Description: Manage SunGrow Inverter (ModBus TCP)
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Sensor
   Platform: Generic
   Param1: STR "IP Address"
   Param2: INT "TCP Port"
   Param3: INT "ModBus device id"
--]]

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local ipaddress = params[4]
local port = params[5]
local modbusdevid = params[6]

if not ipaddress or not port or not modbusdevid then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

libOpen('libserial.so.1.0')
libOpen('fieldbus.so.1.0')
libOpen('modbus.so.1.0')
libOpen('bin_ops.so.1.0')

local unit32swap = 'FIELDBUS_DATA_FMT_INT32_CDAB'
local unit16 = 'FIELDBUS_DATA_FMT_INT16_AB'
local interval = 1000
local noScale = 1

if modbusSungrowInstance == nil then
    modbusSungrowInstance = modbusTcpOpen (string.format('%s:%d', ipaddress, port))

    if (modbusSungrowInstance == -1) then
        ERROR('modbus open failed')
    return
    end

    modbusSungrowInstanceCount = 1
else
    modbusSungrowInstanceCount = modbusSungrowInstanceCount + 1
end

local dev = modbusRegisterDevice (modbusSungrowInstance, modbusdevid)

if dev == -1 then
    ERRORF('Register device %d failed', modbusdevid)
    return
end

-- DC Watts
modbusDefineInputRegisterRange(dev, 5016, 5017, unit32swap, noScale, interval)
modbusDefineInputRegister(dev, 5016, sensorID .. '_total_dc_power_r5016')

-- Phase Voltage AN
modbusDefineInputRegisterRange(dev, 5018, 5018, unit16, 10, interval)
modbusDefineInputRegister(dev, 5018, sensorID .. '_a_b_line_voltage_phase_a_voltage_r5018')

-- PF
modbusDefineInputRegisterRange(dev, 5034, 5034, unit16, 1000, interval)
modbusDefineInputRegister(dev, 5034, sensorID .. '_power_factor_r5034')

-- Hz
modbusDefineInputRegisterRange(dev, 5035, 5035, unit16, 10, interval)
modbusDefineInputRegister(dev, 5035, sensorID .. '_grid_frequency_r5035')

-- Total Power
modbusDefineInputRegisterRange(dev, 13021, 13021, unit16, noScale, interval)
modbusDefineInputRegister(dev, 13021, sensorID .. '_battery_power_r13021')

-- State of Charge
modbusDefineInputRegisterRange(dev, 13022, 13022, unit16, 10, interval)
modbusDefineInputRegister(dev, 13022,  sensorID .. '_battery_level_r13022')

-- Amps
modbusDefineInputRegisterRange(dev, 13030, 13030, unit16, 10, interval)
modbusDefineInputRegister(dev, 13030, sensorID .. '_phase_a_current_r13030')

-- Watts
modbusDefineInputRegisterRange(dev, 13033, 13033, unit32swap, noScale, interval)
modbusDefineInputRegister(dev, 13033, sensorID .. '_total_active_power_r13033')

-- Running State
modbusDefineInputRegisterRange(dev, 13000, 13000, unit16, noScale, interval)
modbusDefineInputRegister(dev, 13000, sensorID .. '_running_state_r13000')


----     "Meter connection state" : {
--modbusDefineHoldingRegisterRange(dev, 13075, 13075, unit16, noScale)
----    "Battery Max SOC [%25]" : {
--modbusDefineHoldingRegisterRange(dev, 13057, 13057, unit16, 10)
----     "Battery Maintenance" : {
--modbusDefineHoldingRegisterRange(dev, 13000, 13000, unit16, 1)
----     "Battery Min SOC [%25]" : {
--modbusDefineHoldingRegisterRange(dev, 13058, 13058, unit16, 10)
----     "Battery overvoltage threshold [V]" : {
--modbusDefineHoldingRegisterRange(dev, 13059, 13059, unit16, 10)


----     "Battery SOH [%25]" : {
--modbusDefineInputRegisterRange(dev, 13023, 13023, unit16, 10, interval)
----     "Battery True SOC [%25]" : {
--modbusDefineInputRegisterRange(dev, 13106, 13106, unit16, 1, interval)
----     "Battery Energy Out [kWh]" : {
--modbusDefineInputRegisterRange(dev, 13026, 13026, unit16, 10, interval)
----     "BMS Status" : {
--modbusDefineInputRegisterRange(dev, 13099, 13099, unit16, 1, interval)
----     "Self-Consumption Daily [%25]" : {
--modbusDefineInputRegisterRange(dev, 13028, 13028, unit16, 1, interval)
----     "Battery Average Cell Temp" : {
--modbusDefineInputRegisterRange(dev, 13115, 13115, unit16, 1, interval)
----     "Serial Number" : {
--modbusDefineInputRegisterRange(dev, 4989, 4989 + 15, string_t, 1, interval)
----     "Inverter Alarm" : {
--modbusDefineInputRegisterRange(dev, 13049, 13049, unit32, 1, interval)
----     "NominalPower" : {
--modbusDefineInputRegisterRange(dev, 5000, 5000, unit16, 10, interval)
----     "AC Reactive Power [var]" : {
--modbusDefineInputRegisterRange(dev, 5032, 5033, unit32swap, 1, interval)
----     "BMS Cycle Count" : {
--modbusDefineInputRegisterRange(dev, 13110, 13110, unit16, 1, interval)
----     "Battery Current [A]" : {
--modbusDefineInputRegisterRange(dev, 13020, 13020, unit16, 10, interval)
----     "Daily Direct Energy Consumption" : {
--modbusDefineInputRegisterRange(dev, 13016, 13016, unit16, 10, interval)
----     "DC Energy Total [kWh]" : {
--modbusDefineInputRegisterRange(dev, 13002, 13003, unit32swap, 10, interval)
----     "Battery Average Cell Voltage [V]" : {
--modbusDefineInputRegisterRange(dev, 13111, 13111, unit16, 1, interval)
----     "Battery Temp [C]" : {
--modbusDefineInputRegisterRange(dev, 13024, 13024, unit16, 10, interval)
----     "Battery Voltage [V]" : {
--modbusDefineInputRegisterRange(dev, 13019, 13019, unit16, 10, interval)
----     "AC Export Energy [kWh]" : {
--modbusDefineInputRegisterRange(dev, 13005, 13006, unit32swap, 10, interval)
----     "Direct Energy Consumption" : {
--modbusDefineInputRegisterRange(dev, 13017, 13018, unit32swap, 10, interval)
----     "Battery Energy In [kWh]" : {
--modbusDefineInputRegisterRange(dev, 13012, 13013, unit32swap, 10, interval)
----     "AC Export Energy Daily [kWh]" : {
--modbusDefineInputRegisterRange(dev, 6564, 6564, unit16, 10, interval)
----     "System State" : {
--modbusDefineInputRegisterRange(dev, 12999, 12999, unit16, 1, interval)


deviceControl.loopDeviceThread(vendorID, deviceID, sensorID)

modbusSungrowInstanceCount = modbusSungrowInstanceCount - 1
modbusUnregisterDevice (dev)

if (modbusSungrowInstanceCount == 0) then
    modbusClose (modbusSungrowInstance)
    modbusSungrowInstance = nil
end

