-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--[[ METADATA
   DeviceName: TOSR0X relay board
   Description: Set tosr04/tosr08 relays connected by serial link
   Version: 1.0.0
   Author: Emutex LTD
   AuthorURI: http://emutex.com
   Type: Actuator
   Platform: Generic
   Min: 0
   Max: 1
   Param1:  STR "Serial port"  "/dev/ttyUSB0"
   Param2:  INT "Relay number" "1"
--]]

--! \file
--! \brief Script to set tosr04 relays connected by serial link

libOpen('libserial.so.1.0')

local deviceControl = require 'utils/device_control'
local utils = require 'utils/utils'

local params = { ... }
local vendorID = params[1]
local deviceID = params[2]
local sensorID = params[3]
local serport = params[4]
local relayNum = tonumber(params[5])

if not serport or not relayNum then
    ERRORF ('%s invalid hwaddr for %q', utils.getScriptName (), sensorID)
    return
end

if relayNum < 1 or relayNum > 8 then
    ERRORF ('Invalid relay number %q specified for tosr04_tty %q', relayNum, sensorID)
    return
end

local function relayOutput (tag, data)
    local cmds

    data = tonumber(data) or 0
    local port = serialOpen(serport, 9600)

    if port < 0 then
        WARN ('Failed to open ' .. serport)
        return
    end


    if data == 0 then
        cmds = 'opqrstuv'
    else
        cmds = 'efghijkl'
    end

    local out = cmds:sub(relayNum, relayNum)
    NOTIFYF ('Sending %s to port %s', out, serport)
    serialWrite(port, out, 1)
    serialClose(port)
end

-- eventHandler (sensorID, relayOutput)

deviceControl.loopActuatorThread(vendorID, deviceID, sensorID, relayOutput)

-- eventHandlerClear (sensorID)

