-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--- A class to read the ADXL 345 accelerometer through I2C
-- @classmod Adxl345

local i2c = require 'drivers/i2c'
libOpen('bin_ops.so.1.0')

local Adxl345 = {}
Adxl345.__index = Adxl345

--- Creates an ADXL345 object and establishes an I2C connection.
-- @tparam int bus I2C bus number where the ADXL345 is connected
-- @tparam int addr I2C address of the ADXL345
-- @treturn Adxl345 the created object
function Adxl345.create(bus, addr)
    local adxlObj = {}
    setmetatable(adxlObj,Adxl345)

    assert(bus, 'Invalid I2C bus for adxl345')
    assert(addr, 'Invalid I2C address for adxl345')

    adxlObj.i2c = i2c.create(bus, addr)
    adxlObj.MEASURE          = 0x08
    adxlObj.AXES_DATA        = 0x32
    adxlObj.DATA_FORMAT      = 0x31
    adxlObj.BW_RATE          = 0x2C
    adxlObj.POWER_CTL        = 0x2D
    adxlObj.FULL_RES         = 0x08
    adxlObj.BW_RATE_1600HZ   = 0x0F
    adxlObj.BW_RATE_800HZ    = 0x0E
    adxlObj.BW_RATE_400HZ    = 0x0D
    adxlObj.BW_RATE_200HZ    = 0x0C
    adxlObj.BW_RATE_100HZ    = 0x0B
    adxlObj.BW_RATE_50HZ     = 0x0A
    adxlObj.BW_RATE_25HZ     = 0x09
    adxlObj.RANGE_2G         = 0x00
    adxlObj.RANGE_4G         = 0x01
    adxlObj.RANGE_8G         = 0x02
    adxlObj.RANGE_16G        = 0x03

    return adxlObj
end

function Adxl345:enableMeasurement()
    self.i2c:writeByte(self.POWER_CTL, self.MEASURE)
end

function Adxl345:setBandwidth(rate)
    self.i2c:writeByte(self.BW_RATE, rate)
end

function Adxl345:setRange(range_flag)
    local value = binOr(range_flag, self.FULL_RES)
    self.i2c:writeByte(self.DATA_FORMAT, value)
end

--- Returns the measured values for all 3 axes.
--  @treturn int acceleration of the x-axis
--  @treturn int acceleration of the y-axis
--  @treturn int acceleration of the z-axis
function Adxl345:getAxes()
    local function normalise(val)
        val = binAnd(val, 0x1FFF) -- remove MSB
        if (val > 0xFFF) then
            val = 0x1FFF - val
        end
        return val
    end

    local x = self.i2c:readWord(self.AXES_DATA)
    local y = self.i2c:readWord(self.AXES_DATA + 2)
    local z = self.i2c:readWord(self.AXES_DATA + 4)

    if x == nil or y == nil or z == nil then
        return nil
    end

    x = normalise(x)
    y = normalise(y)
    z = normalise(z)

    return x, y, z
end

return Adxl345
