-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local muxing = require "drivers/edison/muxing"
local utils = require "utils/utils"

local PWM = {}
PWM.__index = PWM

local NSEC_PER_SEC = 1000000000

function PWM.create(pwmNum)
    local pwmObj = {}
    setmetatable(pwmObj, PWM)

    if not muxing.muxPWM(pwmNum) then
        return nil
    end

    pwmObj.SYS_PATH = string.format("/sys/class/pwm/pwmchip0/pwm%s", pwmNum)
    pwmObj.ENABLE_PATH = string.format("%s/enable", pwmObj.SYS_PATH)
    pwmObj.PERIOD_PATH = string.format("%s/period", pwmObj.SYS_PATH)
    pwmObj.DUTY_PATH = string.format("%s/duty_cycle", pwmObj.SYS_PATH)
    pwmObj.DEFAULT_FREQ = 25000
    pwmObj._enabled = false
    pwmObj._duty = 0
    pwmObj._period = 0
    pwmObj:setFrequency(pwmObj.DEFAULT_FREQ)

    return pwmObj
end

function PWM:enable()
    self._enabled = utils.writeToFile(self.ENABLE_PATH, 1)
    return self._enabled
end

function PWM:disable()
    self._enabled = not utils.writeToFile(self.ENABLE_PATH, 0)
    return not self._enabled
end

function PWM:setDuty(dutyPercent)
    if dutyPercent > 100 then
        dutyPercent = 100
    elseif dutyPercent < 0 then
        dutyPercent = 0
    end

    local dutyNsec = math.floor((dutyPercent / 100) * self._period)
    if not utils.writeToFile(self.DUTY_PATH, dutyNsec) then
        return false
    end
    self._duty = dutyNsec

    return true
end

function PWM:setFrequency(frequencyHz)
    local periodNsec = math.floor(NSEC_PER_SEC / frequencyHz)
    if not utils.writeToFile(self.PERIOD_PATH, periodNsec) then
        return false
    end
    self._period = periodNsec

    return true
end

function PWM:getDuty()
    return self._duty / self._period
end

function PWM:getFrequency()
    return  NSEC_PER_SEC / self._period
end

function PWM:isEnabled()
    return self._enabled
end

return PWM
