-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--- Provides functionality to communicate over an I2C bus.
-- @classmod i2c

libOpen ('i2c.so.1.0')

local i2c = {}
i2c.__index = i2c

local function execute(cmd)
    local file = assert(io.popen(cmd, 'r'))
    local output = file:read('*all')
    file:close()
    return output:gsub('^%s*(.-)%s*$', '%1')
end

--- Creates an I2C object.
-- @tparam int bus I2C bus number
-- @tparam int addr I2C address
function i2c.create(bus, addr)
    local i2cObj = {}
    setmetatable(i2cObj,i2c)

    bus = tonumber(bus)
    i2cObj.addr = tonumber(addr)

    assert(bus, string.format("I2C bus has to be a number, got '%s'", bus))
    assert(i2cObj.addr, string.format("I2C address has to be a number, got '%s'", addr))

    i2cObj.bus = i2cOpen (bus)
    if i2cObj.bus < 0 then
        return nil
    end

    return i2cObj
end

function i2c:readByte (reg)
    return i2cReadByte (self.bus, self.addr, reg)
end

function i2c:readWord (reg)
    return i2cReadWord (self.bus, self.addr, reg)
end

-- NOTE - this is not portable across all platforms
-- (e.g. not supported by UP board)
-- Consider using readRaw() below instead
function i2c:readBlock (reg)
    return i2cReadBlock (self.bus, self.addr, reg)
end

function i2c:readRaw (len)
    return i2cReadRaw (self.bus, self.addr, len)
end

function i2c:writeByte(reg, val)
    return i2cWriteByte (self.bus, self.addr, reg, val)
end

function i2c:writeWord(reg, val)
    return i2cWriteWord (self.bus, self.addr, reg, val)
end

-- NOTE - this is not portable across all platforms
-- (e.g. not supported by UP board)
-- Consider using writeRaw() below instead
function i2c:writeBlock (reg, val)
    return i2cWriteBlock (self.bus, self.addr, reg, val)
end

function i2c:writeRaw (reg, val)
    return i2cWriteRaw (self.bus, self.addr, reg, val)
end

return i2c
