-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

libOpen('libserial.so.1.0')
libOpen('fieldbus.so.1.0')
libOpen('modbus.so.1.0')

local RAYLEIGH = {}
local intDataFormat = 'FIELDBUS_DATA_FMT_INT16_AB'
local floatDataFormat = 'FIELDBUS_DATA_FMT_FLOAT32'
local interval = 1000
local noScale = 1

function RAYLEIGH.new(id)
    if not id then
        ERRORF('RAYLEIGH meter expects valid id')
        return
    end

    local ri = {}
    setmetatable(ri, { __index = RAYLEIGH })
    ri.id = id
    return ri
end

function RAYLEIGH:openTcp(ip, port, modbusdevid)
    if not ip or not port or not modbusdevid then
        ERRORF('RAYLEIGH meter (tcp) %s got invalid params', self.id)
        return
    end

    if not _G.__modbusTcp then
        local modbus = modbusTcpOpen(string.format('%s:%d', ip, port))
        if modbus == -1 then
            ERRORF('RAYLEIGH meter (tcp) failed openning %s:%s', ip, port)
            return
        end
        _G.__modbusTcp = modbus
        _G.__modbusCountTcp = 1
    else
        _G.__modbusCountTcp = _G.__modbusCountTcp + 1
    end

    local dev = modbusRegisterDevice(_G.__modbusTcp, modbusdevid)
    if dev == -1 then
        ERRORF('RAYLEIGH meter (rtu) failed registering device %s:%s id %s',
            ip, port, modbusdevid)
        return
    end
    self.dev = dev

    return true
end

function RAYLEIGH:openRtu(path, baudrate, modbusdevid)
    if not path or not baudrate or not modbusdevid then
        ERRORF('RAYLEIGH meter (rtu) %s got invalid params', self.id)
        return
    end

    if not _G.__modbusRtu then
        local modbus = modbusRtuOpen(path, baudrate)
        if modbus == -1 then
            ERRORF('RAYLEIGH meter (rtu) failed openning %s', path)
            return
        end
        _G.__modbusRtu = modbus
        _G.__modbusCountRtu = 1
    else
        _G.__modbusCountRtu = _G.__modbusCountRtu + 1
    end

    local dev = modbusRegisterDevice(_G.__modbusRtu, modbusdevid)
    if dev == -1 then
        ERRORF('RAYLEIGH meter (rtu) failed registering device %s id %s',
            path, modbusdevid)
        return
    end
    self.dev = dev

    return true
end

function RAYLEIGH:defineRegisters()
    -- Watts
    modbusDefineInputRegisterRange(self.dev, 12, 13, floatDataFormat, noScale, interval)
    modbusDefineInputRegister(self.dev, 12, self.id .. '_active_power_r12')

    -- Total Watt-hours Imported
    modbusDefineInputRegisterRange(self.dev, 72, 73, floatDataFormat, noScale, interval)
    modbusDefineInputRegister(self.dev, 72, self.id .. '_import_active_energy_r72')

    -- Total Watt-hours Exported
    modbusDefineInputRegisterRange(self.dev, 74, 75, floatDataFormat, noScale, interval)
    modbusDefineInputRegister(self.dev, 74, self.id .. '_export_active_energy_r74')

    ---- Total Active
    --modbusDefineInputRegisterRange(self.dev, 342, 343, floatDataFormat, noScale, interval)
    --modbusDefineInputRegister(self.dev, 342, self.id .. '_TotalActive')
    --
    ---- Measure Mode
    --modbusDefineHoldingRegisterRange(self.dev, 463776, 463776, intDataFormat, noScale)
    --modbusDefineHoldingRegister(self.dev, 463776, self.id .. '_MeasureMode')
    --
    ---- Voltage
    --modbusDefineInputRegisterRange(self.dev, 0, 1, floatDataFormat, noScale, interval)
    --modbusDefineInputRegister(self.dev, 0, self.id .. '_Voltage')
    --
    ---- AC Current
    --modbusDefineInputRegisterRange(self.dev, 6, 7, floatDataFormat, noScale, interval)
    --modbusDefineInputRegister(self.dev, 6, self.id .. '_ACCurrent')
end

function RAYLEIGH:closeTcp()
    if self.dev and self.dev > 0 then
        modbusUnregisterDevice(self.dev)
        _G.__modbusCountTcp = _G.__modbusCountTcp - 1
    end

    if _G.__modbusCountTcp == 0 then
        modbusClose(_G.__modbusTcp)
        _G.__modbusTcp = nil
    end
end

function RAYLEIGH:closeRtu()
    if self.dev and self.dev > 0 then
        modbusUnregisterDevice(self.dev)
        _G.__modbusCountRtu = _G.__modbusCountRtu - 1
    end

    if _G.__modbusCountRtu == 0 then
        modbusClose(_G.__modbusRtu)
        _G.__modbusRtu = nil
    end
end

return RAYLEIGH
