-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local utils = require "utils/utils"

local LRFSocket = {}
LRFSocket.__index = LRFSocket

function LRFSocket.create(bus, remoteID, socketNumber)
    local socketObj = {}
    setmetatable(socketObj, LRFSocket)

    socketObj.MSG_LENGTH = 0x0A
    socketObj.MSG_TYPE = 0x14
    socketObj.MSG_SUBTYPE = 0x00
    socketObj.MSG_OFF = 0x00
    socketObj.MSG_ON = 0x01
    socketObj.MSG_LEVEL = 0x00
    socketObj.MSG_SIGNAL_LEVEL = 0x70

    if type(bus) ~= "number" or bus < 0 then
        ERRORF ("LightwaveRF socket init failed, invalid bus")
        return nil
    end

    socketObj.bus = bus
    socketObj.remoteID = remoteID
    socketObj.socketNumber = socketNumber
    socketObj.remoteIDBytes = utils.hexToBytes(socketObj.remoteID)
    socketObj.socketNumberBytes = utils.hexToBytes(socketObj.socketNumber)
    socketObj.ID = tonumber(socketObj.remoteID .. socketObj.socketNumber, 16)

    if not socketObj.remoteIDBytes or not socketObj.socketNumberBytes then
        ERRORF ("LightwaveRF socket init failed, invalid hardware address")
        return nil
    end

    socketObj.tag = string.format("lightwaverf-socket-%s-%s", socketObj.remoteID, socketObj.socketNumber)
    socketObj.handle = rfxcomRegisterDevice(socketObj.bus, socketObj.ID, 0, 0, socketObj.tag)
    if socketObj.handle < 0 then
        ERRORF ("LightwaveRF socket init failed, could not register device")
        return nil
    end

    return socketObj
end

function LRFSocket:destroy()
    -- TODO function runs into an assertion if bus is closed before
    -- rfxcomUnregisterDevice(self.handle)
end

function LRFSocket:sendMessage(command)
    local payload = {
        self.remoteIDBytes,
        self.socketNumberBytes,
        string.char(command),
        string.char(self.MSG_LEVEL),
        string.char(self.MSG_SIGNAL_LEVEL)
    }

    return rfxcomSendData(self.handle,
        self.MSG_LENGTH,
        self.MSG_TYPE,
        self.MSG_SUBTYPE,
        0x00,
        table.concat(payload, "")) == 0
end

function LRFSocket:pair()
    self:on()
end

function LRFSocket:on()
    return self:sendMessage(self.MSG_ON)
end

function LRFSocket:off()
    return self:sendMessage(self.MSG_OFF)
end

return LRFSocket
