-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--- A class to control the RGB LCD display of the grove pi kit.
-- @classmod LcdRgb

libOpen ('bin_ops.so.1.0')

local i2c = require 'drivers/i2c'

local LcdRgb = {}
LcdRgb.__index = LcdRgb

local RGB_ADDR = 0x62
local RGB_RED_REG = 0x04
local RGB_GREEN_REG = 0x03
local RGB_BLUE_REG = 0x02

local TEXT_ADDR = 0x3e
local TEXT_CMD_REG = 0x80
local TEXT_CHAR_REG = 0x40

local CMD_CLEAR_DISP = 0x01
local CMD_DISP_ON = 0x08
local CMD_NO_CURSOR = 0x04
local CMD_TWO_LINES = 0x28
local CMD_NEW_LINE = 0xc0

--- Creates an ADXL345 object and establishes an I2C connection.
-- @tparam int bus I2C bus number where the RGB LCD display is connected
-- @treturn LcdRgb the created object
function LcdRgb.create(bus)
    local lcdObj = {}
    setmetatable (lcdObj, LcdRgb)

    lcdObj.rgb = i2c.create(bus, RGB_ADDR)
    lcdObj.text = i2c.create(bus, TEXT_ADDR)

    return lcdObj
end

function LcdRgb:setText (text)
    self.text:writeByte (TEXT_CMD_REG, CMD_CLEAR_DISP)
    self.text:writeByte (TEXT_CMD_REG, binOr(CMD_DISP_ON, CMD_NO_CURSOR))
    self.text:writeByte (TEXT_CMD_REG, CMD_TWO_LINES)
    local count = 0
    local row = 0
    for c in text:gmatch('.') do
        if c == '\n' or count == 16 then
            if row ~= 0 then
                break
            end

            row = 1
            count = 0
            self.text:writeByte (TEXT_CMD_REG, CMD_NEW_LINE)
        end

        if c ~= '\n' then
            count = count + 1
            self.text:writeByte (TEXT_CHAR_REG, c:byte())
        end
    end
end

function LcdRgb:setRGB (r,g,b)
    self.rgb:writeByte (0, 0)
    self.rgb:writeByte (1, 0)
    self.rgb:writeByte (0x08, 0xaa)
    self.rgb:writeByte (RGB_RED_REG, r)
    self.rgb:writeByte (RGB_GREEN_REG, g)
    self.rgb:writeByte (RGB_BLUE_REG, b)
end

return LcdRgb
