-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--! \brief A class for controlling the vnl4000 proximity sensor

local i2c = require "drivers/i2c"
libOpen("bin_ops.so.1.0")

local Vnl4000 = {}
Vnl4000.__index = Vnl4000

function Vnl4000.create(bus ,addr)
    local vnlObj = {}
    setmetatable(vnlObj,Vnl4000)

    if not bus or not addr then
        ERROR ("Invalid hardware address for vnl4000 provided")
        return nil
    end

    vnlObj.i2c = i2c.create(bus, addr)
    vnlObj.auxReg1 = 0x85
    vnlObj.auxReg2 = 0x86

    -- Configures ambient light measures - Single conversion mode, 128 averages
    vnlObj.i2c:writeByte(0x84, 0x00)
    msleep(2)
    -- sets IR current in steps of 10mA 0-200mA --> 200mA
    vnlObj.i2c:writeByte(0x83, 0x00)
    msleep(2)
    -- Proximity IR test signal freq, 0-3 - 781.25 kHz
    vnlObj.i2c:writeByte(0x89, 0x02)
    msleep(2)
    -- proximity modulator timing - 129
    vnlObj.i2c:writeByte(0x8A, 0x81)
    msleep(2)

    return vnlObj
end

function Vnl4000:readAmbient()
    local _ = self.i2c:readByte(0x80)

    self.i2c:writeByte(0x80, 0x00)
    self.i2c:writeByte(0x80, 0x10)

    local result = 0x00
    while (result == 0x00) do
        local val = self.i2c:readByte(0x80)
        if val == nil then
            ERROR("Failed to read vnl4000")
            return
        end

        result = binAnd(val, 0x40)
    end
    local aux1 = self.i2c:readByte(self.auxReg1)
    local aux2 = self.i2c:readByte(self.auxReg2)

    if aux1 == nil or aux2 == nil then
        ERROR("Failed to read vnl4000")
        return
    end

    local value = binLeft(aux1,8)
    value = value + aux2
    return value
end

return Vnl4000
