-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

-- This script is an example to use the query building mechanisms
-- from the db/ directory.

local query = require "db/query"

-- init database
local dbengine = getSystemConfigValue("dbengine","sqlite3")

if (dbengine == "sqlite3") then
    NOTIFY ("Using sqlite3 as DB engine")
    libOpen ("vxf_sqlite3.so.1.0")

    if dbInit () ~= true then
        FATAL("Failed to open sqlite db")
    end
elseif (dbengine == "mysql")  then
    local dbpasswd = getSystemConfigValue("dbpasswd","ubiworx123")
    NOTIFY ("Using mysql as DB engine")
    libOpen ("vxf_mysql.so.1.0")

    if dbInit ("root", dbpasswd) ~= true then
        FATAL("Failed to open mysql db")
    end
else
    FATAL(string.format("Unknown dbengine: '%s'", dbengine))
end

-- #########################
--    Create Query Strings
-- #########################

local myQuery = query.
    SELECT("keepalive", "longitude", "latitude", "mobile"):
    FROM("device"):
    WHERE("vendor_ID", "=", 4):
    OR("device_ID", "=", "leaf1"):
    toString()

print(myQuery)

myQuery = query.
    SELECT("*"):
    FROM("device_config"):
    WHERE("name", "=", "scadaServer"):
    toString()

print(myQuery)

myQuery = query.
    DELETE():
    FROM("device_config"):
    WHERE("name", "=", "scadaServer"):
    toString()

print(myQuery)

myQuery = query.
    DELETE():
    FROM("device_config"):
    toString()

print(myQuery)

myQuery = query.
    INSERT_INTO("device_config"):
    VALUES(4, "leaf1", "mqttClientServer", "broker.ubiworx.com"):
    toString()

print(myQuery)

myQuery = query.
    INSERT_INTO("device"):
    COLUMNS("vendor_ID", "device_ID", "MAC"):
    VALUES(4, "leaf1", "00:00:00:00:00:00"):
    toString()

print(myQuery)

myQuery = query.
    UPDATE("device_config"):
    SET({value="broker.ubiworx.com"}):
    WHERE("name", "=", "mqttClientBroker"):
    toString()

print(myQuery)

-- #########################
--      Execute Query
-- #########################

local queryResult = query.
    SELECT("*"):
    FROM("device"):
    execute()

if queryResult.count > 0 then
    for row, columns in ipairs(queryResult.data) do
        print(string.format("row %s", row))
        print("===========")
        for columnName, value in pairs(columns) do
            if not value then
                value = "NULL"
            end
            print(string.format("%s: %s", columnName, value))
        end
    end
end
