-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

-- initialise extended logging functions
require "utils/log"
local config = require "utils/config"
local utils = require "utils/utils"
local query = require "db/query"

libOpen ("mqtt.so.1.0")

if getSystemConfigValue('isServer', 'No') == 'Yes' then
    NOTIFY ('Starting as server')

    local dbpasswd = getSystemConfigValue('dbpasswd', 'ubiworx123')
    libOpen ('vxf_mysql.so.1.0')

    if not dbInit ('root', dbpasswd) then
        FATAL('Failed to open mysql db')
    end

    -- deactivate all devices
    query.UPDATE('device'):
        SET({ active = 0 }):
        execute()

    libOpen ('histdb.so.1.0')

    NOTIFY ("Launching server announce thread")
    thread ("server/announce_server")

    NOTIFY ("Launching transactions watcher thread")
    thread ("lite/transactions_watcher")

    return
end

NOTIFY ('Starting as client')
libOpen ('vxf_sqlite3.so.1.0')

local autoTrans = tonumber(getSystemConfigValue('automaticTransactions', '1'))
if not dbInit (autoTrans) then
    FATAL('Failed to open sqlite db')
end

local device = query.
    SELECT('vendor_ID', 'device_ID'):
    FROM('device'):
    execute()

if device.count ~= 1 then
    FATALF('Invalid database setup, found %d records', device.count)
end

local vendorID = device.data[1].vendor_ID
local deviceID = device.data[1].device_ID

-- activate this device
query.UPDATE('device'):
    SET({ active = 1 }):
    WHERE('vendor_ID', '=', vendorID):
    AND('device_ID', '=', deviceID):
    execute()

utils.writeToFile("/var/run/vendor.id", vendorID)
utils.writeToFile("/var/run/device.id", deviceID)

config.refresh(vendorID, deviceID)

local storeFwd = require "client/store_forward"
local osPackages = require "client/os_packages"
local mqttClient = mqttNew(vendorID, deviceID, true)

osPackages.init (vendorID, deviceID)
storeFwd.setMQTTClient(mqttClient)

if not config.hasRTC(vendorID, deviceID) then
    storeFwd.purgeAllSamples(vendorID, deviceID)
end

NOTIFY ("Launching logrequest thread")
thread ("client/logrequest", vendorID, deviceID)

NOTIFY ("Launching data samples thread")
thread ("client/samples", vendorID, deviceID, mqttClient)

NOTIFY ("Launching actuators thread")
thread ("client/actuators", vendorID, deviceID, mqttClient)

NOTIFY ("Launching rules thread")
thread ("client/rules", vendorID, deviceID, mqttClient)

NOTIFY ("Launching client announce thread")
thread ("client/announce_client", vendorID, deviceID, mqttClient)

if config.scadaEnabled(vendorID, deviceID) then
    NOTIFY ("Launching scada thread")
    thread ("client/scada", vendorID, deviceID)
end
