-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

--- A queue implementation using the Lua built-in tables.
-- @classmod Queue

local Queue = {}
Queue.__index = Queue

--- Construct an empty queue object
-- @treturn Queue
function Queue.create ()
    local obj = {}
    setmetatable (obj, Queue)

    obj._first = 0
    obj._last = -1

    return obj
end

--- Push a new element at the left
-- @tparam variant element to be added
function Queue:pushLeft (element)
    local first = self._first - 1
    self._first = first
    self[first] = element
end

--- Push a new element at the right
-- @tparam variant element to be added
function Queue:pushRight (element)
    local last = self._last + 1
    self._last = last
    self[last] = element
end

--- Pop an element from the left
-- @treturn variant element at the left or nil if queue empty
function Queue:popLeft ()
    if self:isEmpty () then
        return nil
    end

    local first = self._first
    local element = self[first]
    self[first] = nil
    self._first = first + 1
    return element
end

--- Pop an element from the right
-- @treturn variant element at the right or nil if queue empty
function Queue:popRight ()
    if self:isEmpty () then
        return nil
    end

    local last = self._last
    local element = self[last]
    self[last] = nil
    self._last = last - 1
    return element
end

--- Return if the queue is empty
-- @treturn boolean
function Queue:isEmpty ()
    return self._first > self._last
end

return Queue
