-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local utils = require "utils/utils"

local self = {}

local scriptsMetadataCache = nil

self.type = {}

self.type["sensor"] = {   --Device type name in lowercase
    typeIdx   = 0,        --Legacy type
    sensor    = true,     --This is a sensor
    actuator  = false,    --This is an actuator
    thread    = true,     --Has device thread
    store     = true,     --Store values
    sendEvery = true,     --Send values to the broker every sample rate
    sendAsync = false     --Send values to the broker every time a new value comes
}
self.type["actuator"] = {
    typeIdx   = 1,
    sensor    = false,
    actuator  = true,
    thread    = true,
    store     = false,
    sendEvery = false,
    sendAsync = false
}
self.type["localsensor"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = false,
    store     = true,
    sendEvery = true,
    sendAsync = false
}
self.type["localactuator"] = {
    typeIdx   = 1,
    sensor    = false,
    actuator  = true,
    thread    = false,
    store     = false,
    sendEvery = false,
    sendAsync = false
}
self.type["localasyncsensor"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = false,
    store     = true,
    sendEvery = false,
    sendAsync = true
}
self.type["ondemandsensor"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = true,
    store     = false,
    sendEvery = false,
    sendAsync = false
}
self.type["storelesssensor"]  = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = true,
    store     = false,
    sendEvery = true,
    sendAsync = false
}
self.type["asyncsensor"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = true,
    store     = true,
    sendEvery = false,
    sendAsync = true
}
self.type["node"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = true,
    store     = false,
    sendEvery = false,
    sendAsync = false
}
self.type["interface"] = {
    typeIdx   = 0,
    sensor    = true,
    actuator  = false,
    thread    = true,
    store     = false,
    sendEvery = false,
    sendAsync = false
}

local function hwtype2type(hwtype)
    assert(type(hwtype) == "string", "hwtype is not a string")
    local devType = self.getMetadataForHwtype(hwtype)
    if (type(devType) ~= "table") then
        return nil
    end
    devType = devType.type
    if type(devType) == "string" then
        return string.lower(devType)
    else
        return nil
    end
end

function self.isSensor(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].sensor
end

function self.isActuator(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].actuator
end

function self.hasThread(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].thread
end

function self.hasStorage(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].store
end

function self.hasSampleRate(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].sendEvery
end

function self.isAsynchronous(hwtype)
    local devType = hwtype2type(hwtype)
    if not devType then
        ERRORF("metadata not found for '%s'", hwtype)
        return nil
    end
    return self.type[devType].sendAsync
end

function self.isValidType(hwtype)
    return (hwtype2type(hwtype) ~= nil)
end

function self.getLocalScriptsMetadata()
    if scriptsMetadataCache then
        return scriptsMetadataCache
    end

    local scriptsMetadata = {
        version    = utils.METADATA_VERSION,
        interfaces = {},
        nodes      = {},
        sensors    = {},
    }

    local deviceScriptsDir = getScriptDir() .. "/devices"
    local f = io.popen(string.format("ls -1 %q", deviceScriptsDir))
    if not f then
        ERROR("Failed to get deviceScripts, could not list scripts")
        return nil
    end

    for line in f:lines() do
        local name, count = line:gsub("%.lua", "")
        if count == 1 then
            local metadata = utils.getScriptMetadata(deviceScriptsDir .. "/" .. line)
            scriptsMetadata.sensors[name] = utils.mergeTables(metadata, self.type[string.lower(metadata.type)])
        end
    end
    f:close()

    scriptsMetadataCache = scriptsMetadata

    return scriptsMetadata
end

function self.getMetadataForHwtype(hwtype)
    return self.getLocalScriptsMetadata().sensors[hwtype]
end

function self.legacyHwType2HwType(hwtype, typeIdx)
    if typeIdx == nil then
        return hwtype
    end

    local conversion = {
        gpio = {},
        gpio_edison = {}
    }
    conversion['local'] = {}

    conversion['gpio'][0]        = 'gpi'
    conversion['gpio'][1]        = 'gpo'
    conversion['gpio_edison'][0] = 'gpi_edison'
    conversion['gpio_edison'][1] = 'gpo_edison'
    conversion['local'][0]       = 'localsensor'
    conversion['local'][1]       = 'localactuator'

    if not conversion[hwtype] then
        return hwtype
    end

    return conversion[hwtype][tonumber(typeIdx)]
end

function self.numifySensorTable(sensorTable)
    sensorTable.vendor_ID = tonumber(sensorTable.vendor_ID)
    sensorTable.type = tonumber(sensorTable.type)
    sensorTable.timestamp = tonumber(sensorTable.timestamp)
    sensorTable.longitude = tonumber(sensorTable.longitude)
    sensorTable.latitude = tonumber(sensorTable.latitude)
    sensorTable.sampleRate = tonumber(sensorTable.sampleRate)
    sensorTable.origin = tonumber(sensorTable.origin)
    sensorTable.mobile = tonumber(sensorTable.mobile) or 0
    sensorTable.scale = tonumber(sensorTable.scale) or 1
    sensorTable.max = tonumber(sensorTable.max) or 1
    sensorTable.min = tonumber(sensorTable.min) or 0
    sensorTable.offset = tonumber(sensorTable.offset) or 0
end

return self
