-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local tags = require 'utils/tags'
local sunrise = require 'utils/sunrise'

local lastSystagUpdate = {}

local systags = {
    updateInterval = 60000,
}

function systags.update()
    local tm = os.date("*t", os.time())
    local hourDiff = tm.hour - os.date("!%H", os.time())

    if hourDiff < 0 then
        hourDiff = hourDiff + 24
    end

    if tm.sec ~= lastSystagUpdate.sec then
        tagSetNum (tags.dSecond, tm.sec)
        tagSetString (tags.dTime, os.date("%H:%M:%S"))
    end

    if tm.min ~= lastSystagUpdate.min then
        tagSetNum (tags.dMinute, tm.min)
        tagSetNum (tags.dMinuteofday, tm.min + tm.hour * 60)
    end

    if tm.hour ~= lastSystagUpdate.hour then
        tagSetNum (tags.dHour, tm.hour)
    end

    if tm.day ~= lastSystagUpdate.day then
        tagSetNum (tags.dDayofmonth, tm.day)
    end

    if tm.month ~= lastSystagUpdate.month then
        tagSetNum (tags.dMonth, tm.month)
    end

    if tm.year ~= lastSystagUpdate.year then
        tagSetNum (tags.dYear, tm.year)
    end

    if tm.wday ~= lastSystagUpdate.wday then
        tagSetNum (tags.dWeekday, tm.wday)
    end

    if tm.yday ~= lastSystagUpdate.yday then
        tagSetNum (tags.dDayofyear, tm.yday)

        local longitude = tagGetNum(tags.dLongitude)
        local latitude = tagGetNum(tags.dLatitude)

        local day, rise, set = sunrise.get_time({
            lat    = latitude,
            lon    = longitude,
            offset = hourDiff
        })

        local text = os.date("%H:%M:%S", rise)
        NOTIFYF ("Sunrise set to %s",  text)
        tagSetString (tags.dSunrise, text)

        text = os.date("%H:%M:%S", set)
        NOTIFYF ("Sunset set to %s",  text)
        tagSetString (tags.dSunset, text)

        NOTIFYF ("On %d/%d/%d with TZ %d at lat %.4f long %.4f rise=%d set=%d",
                tm.day, tm.month, tm.year, hourDiff, longitude, latitude,
                rise, set)
    end

    lastSystagUpdate = tm
end

return systags
