-- CONFIDENTIAL AND PROPRIETARY INFORMATION

-- Copyright (c) 2013,2016 Emutex Ltd. All rights reserved.
-- This software and documentation contain confidential and
-- proprietary information that is the property of
-- Emutex Ltd. The software and documentation are
-- furnished under a license agreement and may be used
-- or copied only in accordance with the terms of the license
-- agreement. No part of the software and documentation
-- may be reproduced, transmitted, or translated, in any
-- form or by any means, electronic, mechanical, manual,
-- optical, or otherwise, without prior written permission
-- of Emutex Ltd., or as expressly provided by the license agreement.
-- Reverse engineering is prohibited, and reproduction,
-- disclosure or use without specific written authorization
-- of Emutex Ltd. is strictly forbidden.

local utils = require 'utils/utils'

local tags = {
    server = '_server',
    client = '_client',
    announce = 'announce',
    keepalive = 'keepalive',
    clientThread = 'clientThread',
    serverThread = 'serverThread',
    deviceThread = 'deviceThread',
    deviceScriptsUpdated = 'deviceScriptsUpdated',
    sensorTimer = 'sensorTimer',
    clientTimer = 'clientTimer',
    systagsTimer = 'systagsTimer',
    sendAcksTimer = 'sendAcksTimer',
    alarm = '_alarm',
    sendSamplesTimer = 'sendSamplesTimer',
    softwareUpgradeStatus = 'softwareUpgradeStatus',
    dSecond = "$SECOND",
    dMinute = "$MINUTE",
    dMinuteofday = "$MINUTEOFDAY",
    dTime = "$TIME",
    dHour = "$HOUR",
    dDayofmonth = "$DAYOFMONTH",
    dMonth = "$MONTH",
    dYear = "$YEAR",
    dWeekday = "$WEEKDAY",
    dDayofyear = "$DAYOFYEAR",
    dLongitude = "$LONGITUDE",
    dLatitude = "$LATITUDE",
    dSunrise = "$SUNRISE",
    dSunset = "$SUNSET",
    logRequest = "_LOGREQUEST",
    logResponse = "_LOGRESPONSE",
}

function tags.buildUbiworxTag (tagTable)
    return table.concat (tagTable, '.')
end

function tags.extractUbiworxTag (tag)
    return utils.split (tag, '.')
end

function tags.extractSensorTag (topic)
    return topic:match ("([^.]+).([^.]+).([^.]+)")
end

function tags.getClientTag (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.client})
end

function tags.getServerTag (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.server})
end

function tags.getSensorTag (vendorID, deviceID, sensorID)
    return tags.buildUbiworxTag ({vendorID, deviceID, sensorID})
end

function tags.getKeepaliveTimer (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.keepalive})
end

function tags.getClientTimer (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.clientTimer})
end

function tags.getSystagsTimer (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.systagsTimer})
end

function tags.getSendAllSamplesTimer (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.sendSamplesTimer})
end

function tags.getSensorTimer (sensorID)
    return tags.buildUbiworxTag ({sensorID, tags.sensorTimer})
end

function tags.getHandshakeDone (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, 'handshakeDone'})
end

function tags.getActuatorsReconfig ()
    return tags.buildUbiworxTag({'actuators', 'reconfig'})
end

function tags.getSensorsReconfig ()
    return tags.buildUbiworxTag({'sensors', 'reconfig'})
end

function tags.getRulesReconfig ()
    return tags.buildUbiworxTag({'rules', 'reconfig'})
end

function tags.getRulesDelay ()
    return tags.buildUbiworxTag({'rules', 'delay'})
end

function tags.getRulesOneShot ()
    return tags.buildUbiworxTag({'rules', 'oneshot'})
end

function tags.getRulesInitIntegrate ()
    return tags.buildUbiworxTag({'rules', 'initIntegrate'})
end

function tags.getRulesIntegrate ()
    return tags.buildUbiworxTag({'rules', 'integrate'})
end

function tags.getRulesRemoteSubscribe ()
    return tags.buildUbiworxTag({'rules', 'remote', 'subscribe'})
end

function tags.getRulesRemotePublish ()
    return tags.buildUbiworxTag({'rules', 'remote', 'publish'})
end

function tags.getScadaPublish ()
    return tags.buildUbiworxTag({'scada', 'publish'})
end

function tags.getAnnounceClientTimer ()
    return tags.buildUbiworxTag({'announce', 'client', 'timer'})
end

function tags.getClientReconnect ()
    return tags.buildUbiworxTag({'client', 'reconnect'})
end

function tags.getClientThread (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.clientThread})
end

function tags.getClientThreadRunFlag(vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.clientThread,
        'running'})
end

function tags.getServerThread (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.serverThread})
end

function tags.getServerThreadRunFlag(vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.serverThread,
        'running'})
end

function tags.getDeviceThread (vendorID, deviceID, sensorID)
    return tags.buildUbiworxTag ({vendorID, deviceID, sensorID,
        tags.deviceThread})
end

function tags.getDeviceThreadReply (vendorID, deviceID, sensorID)
    return tags.buildUbiworxTag ({vendorID, deviceID, sensorID,
        tags.deviceThread, 'reply'})
end

function tags.getDeviceThreadTimer (vendorID, deviceID, sensorID)
    return tags.buildUbiworxTag ({vendorID, deviceID, sensorID,
        tags.deviceThread, 'timer'})
end

function tags.getDeviceThreadUpdateTimer (vendorID, deviceID, sensorID)
    return tags.buildUbiworxTag ({vendorID, deviceID, sensorID,
        tags.deviceThread, 'update', 'timer'})
end

function tags.getDeviceScriptsUpdated (vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.deviceScriptsUpdated})
end

function tags.getRemoteTopic(vendorID, remoteTopic)
    return string.format("%s.%s", vendorID, remoteTopic)
end

function tags.getRemoteReplyTopic(vendorID, deviceID, sensorID)
    return string.format("%s.%s.%s_remote_reply", vendorID, deviceID, sensorID)
end

function tags.getClientAlarm(vendorID, deviceID)
    return tags.buildUbiworxTag ({vendorID, deviceID, tags.alarm})
end

function tags.getConnectorEvent(vendorID, deviceID)
    return string.format("connector-sample_%s.%s", vendorID ,deviceID)
end

return tags
