#!/bin/bash -e

# Copyright 2015-2016 Emutex Ltd. All rights reserved.

# Store a copy of the log
exec >> >(tee -i /var/log/ubiworx/upgrade.log)
exec 2>&1

#  Upgrade script which processes a command file and writes a status file.
#  Format of command file is : "package=version"
#  Format of response file is : "status package installed-version requested-version"
#  If an upgrade fails, status is set to FAIL

VXRUN_UPGRADE_FILE="/var/run/ubiworx/upgrade"
VXRUN_UPGRADE_STATUS_FILE="/var/run/ubiworx/upgrade_status"

upgrade_check()
{
	rm -f $VXRUN_UPGRADE_STATUS_FILE

	if [ ! -e $VXRUN_UPGRADE_FILE ]; then
		echo "No upgrade fie was found. Exiting..."
		return 0
	fi

	apt-get -qq update
	echo "# status package installed-version requested-version" > $VXRUN_UPGRADE_STATUS_FILE

	for line in `cat $VXRUN_UPGRADE_FILE`; do

		package=`echo $line | awk -F '=' '{ print $1 }'`
		requested_version=`echo $line | awk -F '=' '{ print $2 }'`

		echo "Doing upgrade of $package to $requested_version"

		installed_version=`dpkg-query -s $package | awk '/Version/ { print $2 }'`
		echo "Installed version is $installed_version"

		if [ "$installed_version" == "$requested_version" ]; then
			echo "Package $package is up to date"
			echo "OK   $package $installed_version $requested_version" >> $VXRUN_UPGRADE_STATUS_FILE
		else
			echo "Package $package will be upgraded to version $requested_version"
			if ! apt-get install -y --force-yes $package=$requested_version; then
				echo "FAILED!"
				echo "FAIL $package $installed_version $requested_version" >> $VXRUN_UPGRADE_STATUS_FILE
			else
				echo "OK   $package $requested_version $requested_version" >> $VXRUN_UPGRADE_STATUS_FILE
			fi
		fi
	done

	return 0
}

echo "$(date) Starting upgrade script... "

/etc/init.d/ubiworx stop
sleep 10
# need to wait until ubiworx has completely stopped
if ! upgrade_check; then
	echo "Something went wrong, insert ROLLBACK code here"
fi
/etc/init.d/ubiworx start

